from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.remove_collection_response_params_status import RemoveCollectionResponseParamsStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="RemoveCollectionResponseParams")


@_attrs_define
class RemoveCollectionResponseParams:
    """
    Attributes:
        id (Union[Unset, str]): The collection id
        status (Union[Unset, RemoveCollectionResponseParamsStatus]): The status of the operation
    """

    id: Union[Unset, str] = UNSET
    status: Union[Unset, RemoveCollectionResponseParamsStatus] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, RemoveCollectionResponseParamsStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = RemoveCollectionResponseParamsStatus(_status)

        remove_collection_response_params = cls(
            id=id,
            status=status,
        )

        return remove_collection_response_params



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="WordMatch")


@_attrs_define
class WordMatch:
    """Represents a word match result from word embeddings similarity search

    Attributes:
        id (str): Unique identifier for the word
        language_code (str): The language code of the word
        word (str): The word
    """

    id: str
    language_code: str
    word: str

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        language_code = self.language_code

        word = self.word

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "languageCode": language_code,
                "word": word,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        language_code = d.pop("languageCode")

        word = d.pop("word")

        word_match = cls(
            id=id,
            language_code=language_code,
            word=word,
        )

        return word_match



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.impresso_named_entity_recognition_entity import ImpressoNamedEntityRecognitionEntity


T = TypeVar("T", bound="ImpressoNamedEntityRecognitionResponse")


@_attrs_define
class ImpressoNamedEntityRecognitionResponse:
    """Response of the Impresso NER endpoint

    Attributes:
        model_id (str): ID of the model used for the named entity recognition
        text (str): Text processed for named entity recognition
        timestamp (datetime.datetime): Timestamp of when named entity recognition was performed
        entities (List['ImpressoNamedEntityRecognitionEntity']):
    """

    model_id: str
    text: str
    timestamp: datetime.datetime
    entities: List["ImpressoNamedEntityRecognitionEntity"]

    def to_dict(self) -> Dict[str, Any]:
        model_id = self.model_id

        text = self.text

        timestamp = self.timestamp.isoformat()

        entities = []
        for entities_item_data in self.entities:
            entities_item = entities_item_data.to_dict()
            entities.append(entities_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "modelId": model_id,
                "text": text,
                "timestamp": timestamp,
                "entities": entities,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.impresso_named_entity_recognition_entity import ImpressoNamedEntityRecognitionEntity

        d = src_dict.copy()
        model_id = d.pop("modelId")

        text = d.pop("text")

        timestamp = isoparse(d.pop("timestamp"))

        entities = []
        _entities = d.pop("entities")
        for entities_item_data in _entities:
            entities_item = ImpressoNamedEntityRecognitionEntity.from_dict(entities_item_data)

            entities.append(entities_item)

        impresso_named_entity_recognition_response = cls(
            model_id=model_id,
            text=text,
            timestamp=timestamp,
            entities=entities,
        )

        return impresso_named_entity_recognition_response



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.authentication_response_authentication_payload import AuthenticationResponseAuthenticationPayload


T = TypeVar("T", bound="AuthenticationResponseAuthentication")


@_attrs_define
class AuthenticationResponseAuthentication:
    """
    Attributes:
        strategy (Union[Unset, str]):
        payload (Union[Unset, AuthenticationResponseAuthenticationPayload]):
    """

    strategy: Union[Unset, str] = UNSET
    payload: Union[Unset, "AuthenticationResponseAuthenticationPayload"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        strategy = self.strategy

        payload: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.payload, Unset):
            payload = self.payload.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if strategy is not UNSET:
            field_dict["strategy"] = strategy
        if payload is not UNSET:
            field_dict["payload"] = payload

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.authentication_response_authentication_payload import AuthenticationResponseAuthenticationPayload

        d = src_dict.copy()
        strategy = d.pop("strategy", UNSET)

        _payload = d.pop("payload", UNSET)
        payload: Union[Unset, AuthenticationResponseAuthenticationPayload]
        if isinstance(_payload, Unset):
            payload = UNSET
        else:
            payload = AuthenticationResponseAuthenticationPayload.from_dict(_payload)

        authentication_response_authentication = cls(
            strategy=strategy,
            payload=payload,
        )

        authentication_response_authentication.additional_properties = d
        return authentication_response_authentication

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum
from typing import Literal


class ContentItemMediaType(str, Enum):
    ENCYCLOPEDIA = "encyclopedia"
    MONOGRAPH = "monograph"
    NEWSPAPER = "newspaper"
    RADIO_BROADCAST = "radio_broadcast"
    RADIO_MAGAZINE = "radio_magazine"
    RADIO_SCHEDULE = "radio_schedule"

    def __str__(self) -> str:
        return str(self.value)


ContentItemMediaTypeLiteral = Literal[
    "encyclopedia",
    "monograph",
    "newspaper",
    "radio_broadcast",
    "radio_magazine",
    "radio_schedule",
]



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.find_text_reuse_clusters_base_find_response_pagination import (
        FindTextReuseClustersBaseFindResponsePagination,
    )
    from ..models.text_reuse_cluster import TextReuseCluster


T = TypeVar("T", bound="FindTextReuseClustersBaseFindResponse")


@_attrs_define
class FindTextReuseClustersBaseFindResponse:
    """
    Attributes:
        data (List['TextReuseCluster']):
        pagination (FindTextReuseClustersBaseFindResponsePagination):
    """

    data: List["TextReuseCluster"]
    pagination: "FindTextReuseClustersBaseFindResponsePagination"

    def to_dict(self) -> Dict[str, Any]:
        data = []
        for data_item_data in self.data:
            data_item = data_item_data.to_dict()
            data.append(data_item)

        pagination = self.pagination.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "data": data,
                "pagination": pagination,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.find_text_reuse_clusters_base_find_response_pagination import (
            FindTextReuseClustersBaseFindResponsePagination,
        )
        from ..models.text_reuse_cluster import TextReuseCluster

        d = src_dict.copy()
        data = []
        _data = d.pop("data")
        for data_item_data in _data:
            data_item = TextReuseCluster.from_dict(data_item_data)

            data.append(data_item)

        pagination = FindTextReuseClustersBaseFindResponsePagination.from_dict(d.pop("pagination"))

        find_text_reuse_clusters_base_find_response = cls(
            data=data,
            pagination=pagination,
        )

        return find_text_reuse_clusters_base_find_response



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="SearchBaseFindResponsePagination")


@_attrs_define
class SearchBaseFindResponsePagination:
    """
    Attributes:
        total (int): The total number of items matching the query
        limit (int): The number of items returned in this response
        offset (int): Starting index of the items subset returned in this response
    """

    total: int
    limit: int
    offset: int

    def to_dict(self) -> Dict[str, Any]:
        total = self.total

        limit = self.limit

        offset = self.offset

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "total": total,
                "limit": limit,
                "offset": offset,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        total = d.pop("total")

        limit = d.pop("limit")

        offset = d.pop("offset")

        search_base_find_response_pagination = cls(
            total=total,
            limit=limit,
            offset=offset,
        )

        return search_base_find_response_pagination



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.data_provider_names_item import DataProviderNamesItem


T = TypeVar("T", bound="DataProvider")


@_attrs_define
class DataProvider:
    """A data provider is a partner institution that provides content to Impresso (e.g., libraries, archives, media
    organizations).

        Attributes:
            id (str): The unique identifier of the data provider.
            name (str): The default name of the data provider.
            names (List['DataProviderNamesItem']): Names of the data provider in different languages.
    """

    id: str
    name: str
    names: List["DataProviderNamesItem"]

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        names = []
        for names_item_data in self.names:
            names_item = names_item_data.to_dict()
            names.append(names_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "name": name,
                "names": names,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.data_provider_names_item import DataProviderNamesItem

        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name")

        names = []
        _names = d.pop("names")
        for names_item_data in _names:
            names_item = DataProviderNamesItem.from_dict(names_item_data)

            names.append(names_item)

        data_provider = cls(
            id=id,
            name=name,
            names=names,
        )

        return data_provider



import datetime
from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.collection_access_level import CollectionAccessLevel
from ..types import UNSET, Unset

T = TypeVar("T", bound="Collection")


@_attrs_define
class Collection:
    """Collection details.

    Attributes:
        uid (str): Unique identifier of the collection.
        title (Union[Unset, str]): Title of the collection.
        description (Union[Unset, str]): Description of the collection.
        access_level (Union[Unset, CollectionAccessLevel]): Access level of the collection.
        created_at (Union[Unset, datetime.datetime]): Creation date of the collection.
        updated_at (Union[Unset, datetime.datetime]): Last update date of the collection.
        total_items (Union[Unset, int]): Total number of items in the collection.
    """

    uid: str
    title: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    access_level: Union[Unset, CollectionAccessLevel] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    total_items: Union[Unset, int] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        title = self.title

        description = self.description

        access_level: Union[Unset, str] = UNSET
        if not isinstance(self.access_level, Unset):
            access_level = self.access_level.value

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        total_items = self.total_items

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
            }
        )
        if title is not UNSET:
            field_dict["title"] = title
        if description is not UNSET:
            field_dict["description"] = description
        if access_level is not UNSET:
            field_dict["accessLevel"] = access_level
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if total_items is not UNSET:
            field_dict["totalItems"] = total_items

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        uid = d.pop("uid")

        title = d.pop("title", UNSET)

        description = d.pop("description", UNSET)

        _access_level = d.pop("accessLevel", UNSET)
        access_level: Union[Unset, CollectionAccessLevel]
        if isinstance(_access_level, Unset):
            access_level = UNSET
        else:
            access_level = CollectionAccessLevel(_access_level)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        total_items = d.pop("totalItems", UNSET)

        collection = cls(
            uid=uid,
            title=title,
            description=description,
            access_level=access_level,
            created_at=created_at,
            updated_at=updated_at,
            total_items=total_items,
        )

        return collection



from typing import Any, Callable, Iterator, cast

from pandas import DataFrame, json_normalize

from impresso.api_client.api.images import find_images
from impresso.api_client.api.images import get_image
from impresso.api_client.models.find_images_base_find_response import (
    FindImagesBaseFindResponse,
)
from impresso.api_client.models.find_images_order_by import (
    FindImagesOrderBy,
    FindImagesOrderByLiteral,
)
from impresso.api_client.types import UNSET
from impresso.api_models import Q, BaseFind, Filter, Image
from impresso.data_container import DataContainer, iterate_pages
from impresso.resources.base import DEFAULT_PAGE_SIZE, Resource
from impresso.resources.tools import Embedding
from impresso.structures import AND, OR, DateRange
from impresso.util.error import raise_for_error
from impresso.util.filters import and_or_filter, filters_as_protobuf
from impresso.util.py import get_enum_from_literal
import pandas as pd


# The formatter function needs to be available
def _path_to_image_html(path):
    # Ensure the URL is wrapped in an HTML <img> tag
    return f'<img src="{path}"/>'


class ImageDataFrame(pd.DataFrame):
    @property
    def _constructor(self):
        return ImageDataFrame

    def _repr_html_(self):
        """
        Overrides the default method used by Jupyter/Colab to render DataFrames.
        It forces the display of the styled version.
        """
        # 1. Apply the desired style/formatter
        styler = self.style.format(
            {"previewImage": _path_to_image_html},
        )

        # 2. Return the HTML string generated by the Styler
        return styler.to_html()


def _as_image_df(df: pd.DataFrame) -> ImageDataFrame:
    idf = ImageDataFrame(df)
    cols = idf.columns.tolist()

    if "previewUrl" in cols:
        idf["previewImage"] = idf["previewUrl"]
        cols = idf.columns.tolist()
    # Rearrange columns to make previewImage second
    if "previewImage" in cols:
        cols.remove("previewImage")
        cols.insert(1, "previewImage")
    # Push the URL to the back
    if "previewUrl" in cols:
        cols.remove("previewUrl")
        cols.insert(-1, "previewUrl")
    return cast(ImageDataFrame, idf[cols])


class FindImagesSchema(BaseFind):
    """Schema for the find images response."""

    data: list[Image]


class FindImagesContainer(DataContainer):
    """Response of a find call."""

    def __init__(
        self,
        data: FindImagesBaseFindResponse,
        pydantic_model: type[FindImagesSchema],
        fetch_method: Callable[..., "FindImagesContainer"],
        fetch_method_args: dict[str, Any],
        web_app_search_result_url: str | None = None,
    ):
        super().__init__(data, pydantic_model, web_app_search_result_url)
        self._fetch_method = fetch_method
        self._fetch_method_args = fetch_method_args

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            df = json_normalize(self._data.to_dict()["data"]).set_index("uid")
            return _as_image_df(df)

        return DataFrame()

    def pages(self) -> Iterator["FindImagesContainer"]:
        """Iterate over all pages of results."""
        yield self
        yield from iterate_pages(
            self._fetch_method,
            self._fetch_method_args,
            self.offset,
            self.limit,
            self.total,
        )


class GetImageContainer(DataContainer):

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()
        if len(data):
            df = json_normalize(data).set_index("uid")
            return _as_image_df(df)

        return DataFrame()


class ImagesResource(Resource):
    """Search images in Impresso.

    Examples:
        Search for images by keyword:
        >>> results = images.find(term="war")  # doctest: +SKIP
        >>> print(results.df)  # doctest: +SKIP

        Filter images by date range and newspaper:
        >>> from impresso import DateRange
        >>> date_range = DateRange(start="1900-01-01", end="1910-12-31")
        >>> results = images.find(media_id="GDL", date_range=date_range)  # doctest: +SKIP
        >>> print(results.df)  # doctest: +SKIP

        Search for front page images only:
        >>> results = images.find(is_front=True)  # doctest: +SKIP
        >>> print(results.df)  # doctest: +SKIP

        Search images by visual similarity using embeddings:
        >>> embedding = tools.embed_image("path/to/image.jpg", target="image")  # doctest: +SKIP
        >>> similar_images = images.find(embedding=embedding)  # doctest: +SKIP
        >>> print(similar_images.df)  # doctest: +SKIP

        Get a specific image by its ID:
        >>> image_id = "some-image-id"  # Replace with a real ID
        >>> image = images.get(image_id)  # doctest: +SKIP
        >>> print(image.df)  # doctest: +SKIP
    """

    name = "images"

    def find(
        self,
        term: str | None = None,
        media_id: str | AND[str] | OR[str] | None = None,
        issue_id: str | AND[str] | OR[str] | None = None,
        is_front: bool | None = None,
        date_range: DateRange | None = None,
        visual_content: str | AND[str] | OR[str] | None = None,
        technique: str | AND[str] | OR[str] | None = None,
        communication_goal: str | AND[str] | OR[str] | None = None,
        content_type: str | AND[str] | OR[str] | None = None,
        embedding: Embedding | None = None,
        include_embeddings: bool = False,
        order_by: FindImagesOrderByLiteral | None = None,
        limit: int | None = None,
        offset: int | None = None,
    ) -> FindImagesContainer:
        """Find images in Impresso.

        Args:
            term: The search term for text-based search.
            media_id: Filter by newspaper media ID(s).
            issue_id: Filter by issue ID(s).
            is_front: Filter for front page images only.
            date_range: Filter by date range.
            visual_content: Filter by visual content category.
            technique: Filter by image technique.
            communication_goal: Filter by communication goal.
            content_type: Filter by content type.
            embedding: Image embedding for similarity search. Use `tools.embed_image()` or
                `tools.embed_text()` to generate embeddings from images.
            include_embeddings: Whether to include image embeddings in the response. Defaults to False.
            order_by: Sort order for results.
            limit: Maximum number of results to return per page. Defaults to 100.
            offset: Number of results to skip.

        Returns:
            FindImagesContainer: Data container with the first page of the search results.
        """
        page_limit = limit if limit is not None else DEFAULT_PAGE_SIZE
        embedding_with_limit = f"{embedding}:{page_limit}" if embedding else None

        filters: list[Filter] = []
        if media_id is not None:
            filters.extend(and_or_filter(media_id, "newspaper"))
        if issue_id is not None:
            filters.extend(and_or_filter(issue_id, "issue"))
        if is_front is not None:
            filters.append(Filter(type="isFront", daterange=None))
        if date_range is not None:
            filters.append(
                Filter(
                    type="daterange",
                    q=Q(DateRange._as_filter_value(date_range)),
                    context="exclude" if date_range.inverted else "include",
                    daterange=None,
                )
            )
        if visual_content is not None:
            filters.extend(and_or_filter(visual_content, "image_visual_content"))
        if technique is not None:
            filters.extend(and_or_filter(technique, "image_technique"))
        if communication_goal is not None:
            filters.extend(
                and_or_filter(communication_goal, "image_communication_goal")
            )
        if content_type is not None:
            filters.extend(and_or_filter(content_type, "image_content_type"))
        if embedding_with_limit is not None:
            filters.extend(and_or_filter(embedding_with_limit, "embedding"))

        filters_pb = filters_as_protobuf(filters or [])

        result = find_images.sync(
            client=self._api_client,
            term=term if term is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, FindImagesOrderBy)
                if order_by is not None
                else UNSET
            ),
            limit=page_limit,
            offset=offset if offset is not None else UNSET,
            filters=filters_pb if filters_pb else UNSET,
            include_embeddings=include_embeddings if include_embeddings else UNSET,
        )
        raise_for_error(result)

        return FindImagesContainer(
            cast(FindImagesBaseFindResponse, result),
            FindImagesSchema,
            fetch_method=self.find,
            fetch_method_args={"term": term},
            web_app_search_result_url=_build_web_app_find_images_url(
                base_url=self._get_web_app_base_url(),
                term=term,
            ),
        )

    def get(
        self,
        id: str,
        include_embeddings: bool = False,
    ) -> GetImageContainer:
        """
        Get an image by its id.

        Args:
            id: The id of the image.
            include_embeddings: Whether to include embeddings in the response.

        Returns:
            GetImageContainer: The image data container.
        """
        result = get_image.sync(
            client=self._api_client,
            id=id,
            include_embeddings=include_embeddings,
        )
        raise_for_error(result)

        id_parts = id.split("-")
        issue_id = "-".join(id_parts[:-1])
        article_id = id_parts[-1]

        return GetImageContainer(
            result,
            Image,
            f"{self._get_web_app_base_url()}/issue/{issue_id}/view?articleId={article_id}",
        )

    def get_embeddings(self, id: str) -> list[str]:
        """
        Get the embeddings of an image by its id.

        Args:
            id: The id of the image.

        Returns:
            list[str]: The embeddings of the image if present (every embedding is returned
                in the canonical form: <model>:<base64_embedding>).
        """
        item = self.get(id, include_embeddings=True)
        return item.raw.get("embeddings", []) if item else []


def _build_web_app_find_images_url(
    base_url: str,
    term: str | None = None,
) -> str:
    query_params = {
        "q": term,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    url = f"{base_url}/images"
    return f"{url}?{query_string}" if query_string else url



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="ImpressoNamedEntityRecognitionEntityOffset")


@_attrs_define
class ImpressoNamedEntityRecognitionEntityOffset:
    """
    Attributes:
        start (int): Start offset of the entity in the text
        end (int): End offset of the entity in the text
    """

    start: int
    end: int

    def to_dict(self) -> Dict[str, Any]:
        start = self.start

        end = self.end

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "start": start,
                "end": end,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        start = d.pop("start")

        end = d.pop("end")

        impresso_named_entity_recognition_entity_offset = cls(
            start=start,
            end=end,
        )

        return impresso_named_entity_recognition_entity_offset



import base64
import struct
from typing import List


def embedding_to_vector(embedding: str) -> List[float]:
    """
    Convert a base64-encoded embedding string to an array of floats.

    The embedding string is expected to be in the format: <model>:<base64-encoded vector>
    where the base64-encoded part represents an array of float32 values.

    Args:
        embedding: A string in the format "<model>:<base64-encoded vector>"
                  (e.g., "clip-ViT-B-32:AAAAAAAAAAAAAAAA...")

    Returns:
        A list of float values representing the embedding vector

    Raises:
        ValueError: If the embedding string format is invalid

    Example:
        >>> embedding = "clip-ViT-B-32:AAAAAAAAAAAAAAAA..."
        >>> vector = embedding_to_vector(embedding)
        >>> print(vector)
        [0.0, 0.0, 0.0, ...]
    """
    if ':' not in embedding:
        raise ValueError(
            "Invalid embedding format. Expected '<model>:<base64-encoded vector>'"
        )

    # Split the model prefix from the base64-encoded vector
    _, base64_vector = embedding.split(':', 1)

    # Decode the base64 string to bytes
    vector_bytes = base64.b64decode(base64_vector)

    # Convert bytes to array of float32 values
    # Each float32 is 4 bytes, so we calculate the number of floats
    num_floats = len(vector_bytes) // 4

    # Unpack the bytes as little-endian float32 values
    vector = list(struct.unpack(f'<{num_floats}f', vector_bytes))

    return vector


def vector_to_embedding(vector: List[float], model: str) -> str:
    """
    Convert an array of floats to a base64-encoded embedding string with model prefix.

    Args:
        vector: A list of float values representing the embedding vector
        model: The model identifier to use as prefix (e.g., "clip-ViT-B-32")

    Returns:
        A string in the format "<model>:<base64-encoded vector>"

    Example:
        >>> vector = [0.0, 0.1, 0.2, 0.3]
        >>> embedding = vector_to_embedding(vector, "clip-ViT-B-32")
        >>> print(embedding)
        clip-ViT-B-32:AAAAAAAAAAAAAAAA...
    """
    # Pack the float values as little-endian float32 bytes
    vector_bytes = struct.pack(f'<{len(vector)}f', *vector)

    # Encode the bytes as base64
    base64_vector = base64.b64encode(vector_bytes).decode('ascii')

    # Combine model prefix with base64-encoded vector
    embedding = f"{model}:{base64_vector}"

    return embedding



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="ImpressoEmbeddingResponse")


@_attrs_define
class ImpressoEmbeddingResponse:
    """Body of a response from the Impresso Embedding endpoint

    Attributes:
        embedding (str): Embedding vector, base64-encoded with the model prefix. E.g. <model>:<base64-encoded vector>
    """

    embedding: str

    def to_dict(self) -> Dict[str, Any]:
        embedding = self.embedding

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "embedding": embedding,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        embedding = d.pop("embedding")

        impresso_embedding_response = cls(
            embedding=embedding,
        )

        return impresso_embedding_response



from impresso.client import connect, version  # noqa
from impresso.api_models import *  # noqa
from impresso.structures import (  # noqa
    TermSet,
    AND,
    OR,
    DateRange,
    Fuzzy,
    Soft,
    Exact,
    Partial,
)

__version__ = version



from enum import Enum
from typing import Literal


class AddCollectableItemsFromFiltersNamespace(str, Enum):
    SEARCH = "search"
    TR_PASSAGES = "tr_passages"

    def __str__(self) -> str:
        return str(self.value)


AddCollectableItemsFromFiltersNamespaceLiteral = Literal[
    "search",
    "tr_passages",
]



from enum import Enum
from typing import Literal


class FindTextReusePassagesOrderBy(str, Enum):
    CLUSTERSIZE = "clusterSize"
    DATE = "date"
    LEXICALOVERLAP = "lexicalOverlap"
    SIZE = "size"
    TIMEDIFFERENCEDAY = "timeDifferenceDay"
    VALUE_1 = "-clusterSize"
    VALUE_3 = "-lexicalOverlap"
    VALUE_5 = "-timeDifferenceDay"
    VALUE_7 = "-size"
    VALUE_9 = "-date"

    def __str__(self) -> str:
        return str(self.value)


FindTextReusePassagesOrderByLiteral = Literal[
    "clusterSize",
    "date",
    "lexicalOverlap",
    "size",
    "timeDifferenceDay",
    "-clusterSize",
    "-lexicalOverlap",
    "-timeDifferenceDay",
    "-size",
    "-date",
]



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.content_item_copyright_status import ContentItemCopyrightStatus
from ..models.content_item_media_type import ContentItemMediaType
from ..models.content_item_source_medium import ContentItemSourceMedium
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.content_item_entities_mentions_information import ContentItemEntitiesMentionsInformation
    from ..models.content_item_named_entities_information import ContentItemNamedEntitiesInformation
    from ..models.topic_mention import TopicMention


T = TypeVar("T", bound="ContentItem")


@_attrs_define
class ContentItem:
    """A journal/magazine content item (article, advertisement, etc.)

    Attributes:
        uid (str): The unique identifier of the content item.
        copyright_status (Union[Unset, ContentItemCopyrightStatus]): Copyright status.
        type (Union[Unset, str]): The type of the content item, as present in the OLR provided by the data provider. All
            content items are not characterised by the same set of types.
        source_medium (Union[Unset, ContentItemSourceMedium]): Medium of the source (audio for audio radio broadcasts,
            print for newspapers, typescript for digitised radio bulletin typescripts).
        title (Union[Unset, str]): The title of the content item.
        transcript (Union[Unset, str]): Transcript of the content item.
        entities (Union[Unset, ContentItemNamedEntitiesInformation]): A collection of linked named entities (location,
            person, etc.) present in text.
        mentions (Union[Unset, ContentItemEntitiesMentionsInformation]): A collection of entity mentions (location,
            person, etc.) present in text.
        topics (Union[Unset, List['TopicMention']]): Topics mentioned in the content item.
        embeddings (Union[Unset, List[str]]): Precomputed embeddings for the content item in the format:
            <model_type>:<base64_embedding_vector>.
        transcript_length (Union[Unset, float]): The length of the transcript in characters.
        total_pages (Union[Unset, float]): Total number of pages the item covers.
        language_code (Union[Unset, str]): ISO 639-1 language code of the content item.
        is_on_front_page (Union[Unset, bool]): Whether the content item is on the front page of the publication.
        publication_date (Union[Unset, datetime.datetime]): The publication date of the content item.
        issue_uid (Union[Unset, str]): Unique issue identifier
        country_code (Union[Unset, str]): ISO 3166-1 alpha-2 country code of the content item.
        provider_code (Union[Unset, str]): The code of the data provider.
        media_uid (Union[Unset, str]): Media title alias. Usually a 3 letter code of the media title (newspaper, radio
            station, etc.).
        media_type (Union[Unset, ContentItemMediaType]): The type of the media the content item belongs to.
        has_olr (Union[Unset, bool]): Whether the content item has OCR/OLR data available.
        ocr_quality_score (Union[Unset, float]): OCR quality score of the content item (0 - 1).
        relevance_score (Union[Unset, float]): Relevance score of the content item (0 - 1).
        page_numbers (Union[Unset, List[float]]): Page numbers the content item appears on.
        collection_uids (Union[Unset, List[str]]): Unique identifiers of collections the content item belongs to.
    """

    uid: str
    copyright_status: Union[Unset, ContentItemCopyrightStatus] = UNSET
    type: Union[Unset, str] = UNSET
    source_medium: Union[Unset, ContentItemSourceMedium] = UNSET
    title: Union[Unset, str] = UNSET
    transcript: Union[Unset, str] = UNSET
    entities: Union[Unset, "ContentItemNamedEntitiesInformation"] = UNSET
    mentions: Union[Unset, "ContentItemEntitiesMentionsInformation"] = UNSET
    topics: Union[Unset, List["TopicMention"]] = UNSET
    embeddings: Union[Unset, List[str]] = UNSET
    transcript_length: Union[Unset, float] = UNSET
    total_pages: Union[Unset, float] = UNSET
    language_code: Union[Unset, str] = UNSET
    is_on_front_page: Union[Unset, bool] = UNSET
    publication_date: Union[Unset, datetime.datetime] = UNSET
    issue_uid: Union[Unset, str] = UNSET
    country_code: Union[Unset, str] = UNSET
    provider_code: Union[Unset, str] = UNSET
    media_uid: Union[Unset, str] = UNSET
    media_type: Union[Unset, ContentItemMediaType] = UNSET
    has_olr: Union[Unset, bool] = UNSET
    ocr_quality_score: Union[Unset, float] = UNSET
    relevance_score: Union[Unset, float] = UNSET
    page_numbers: Union[Unset, List[float]] = UNSET
    collection_uids: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        copyright_status: Union[Unset, str] = UNSET
        if not isinstance(self.copyright_status, Unset):
            copyright_status = self.copyright_status.value

        type = self.type

        source_medium: Union[Unset, str] = UNSET
        if not isinstance(self.source_medium, Unset):
            source_medium = self.source_medium.value

        title = self.title

        transcript = self.transcript

        entities: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.entities, Unset):
            entities = self.entities.to_dict()

        mentions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.mentions, Unset):
            mentions = self.mentions.to_dict()

        topics: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.topics, Unset):
            topics = []
            for topics_item_data in self.topics:
                topics_item = topics_item_data.to_dict()
                topics.append(topics_item)

        embeddings: Union[Unset, List[str]] = UNSET
        if not isinstance(self.embeddings, Unset):
            embeddings = self.embeddings

        transcript_length = self.transcript_length

        total_pages = self.total_pages

        language_code = self.language_code

        is_on_front_page = self.is_on_front_page

        publication_date: Union[Unset, str] = UNSET
        if not isinstance(self.publication_date, Unset):
            publication_date = self.publication_date.isoformat()

        issue_uid = self.issue_uid

        country_code = self.country_code

        provider_code = self.provider_code

        media_uid = self.media_uid

        media_type: Union[Unset, str] = UNSET
        if not isinstance(self.media_type, Unset):
            media_type = self.media_type.value

        has_olr = self.has_olr

        ocr_quality_score = self.ocr_quality_score

        relevance_score = self.relevance_score

        page_numbers: Union[Unset, List[float]] = UNSET
        if not isinstance(self.page_numbers, Unset):
            page_numbers = self.page_numbers

        collection_uids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.collection_uids, Unset):
            collection_uids = self.collection_uids

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
            }
        )
        if copyright_status is not UNSET:
            field_dict["copyrightStatus"] = copyright_status
        if type is not UNSET:
            field_dict["type"] = type
        if source_medium is not UNSET:
            field_dict["sourceMedium"] = source_medium
        if title is not UNSET:
            field_dict["title"] = title
        if transcript is not UNSET:
            field_dict["transcript"] = transcript
        if entities is not UNSET:
            field_dict["entities"] = entities
        if mentions is not UNSET:
            field_dict["mentions"] = mentions
        if topics is not UNSET:
            field_dict["topics"] = topics
        if embeddings is not UNSET:
            field_dict["embeddings"] = embeddings
        if transcript_length is not UNSET:
            field_dict["transcriptLength"] = transcript_length
        if total_pages is not UNSET:
            field_dict["totalPages"] = total_pages
        if language_code is not UNSET:
            field_dict["languageCode"] = language_code
        if is_on_front_page is not UNSET:
            field_dict["isOnFrontPage"] = is_on_front_page
        if publication_date is not UNSET:
            field_dict["publicationDate"] = publication_date
        if issue_uid is not UNSET:
            field_dict["issueUid"] = issue_uid
        if country_code is not UNSET:
            field_dict["countryCode"] = country_code
        if provider_code is not UNSET:
            field_dict["providerCode"] = provider_code
        if media_uid is not UNSET:
            field_dict["mediaUid"] = media_uid
        if media_type is not UNSET:
            field_dict["mediaType"] = media_type
        if has_olr is not UNSET:
            field_dict["hasOLR"] = has_olr
        if ocr_quality_score is not UNSET:
            field_dict["ocrQualityScore"] = ocr_quality_score
        if relevance_score is not UNSET:
            field_dict["relevanceScore"] = relevance_score
        if page_numbers is not UNSET:
            field_dict["pageNumbers"] = page_numbers
        if collection_uids is not UNSET:
            field_dict["collectionUids"] = collection_uids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.content_item_entities_mentions_information import ContentItemEntitiesMentionsInformation
        from ..models.content_item_named_entities_information import ContentItemNamedEntitiesInformation
        from ..models.topic_mention import TopicMention

        d = src_dict.copy()
        uid = d.pop("uid")

        _copyright_status = d.pop("copyrightStatus", UNSET)
        copyright_status: Union[Unset, ContentItemCopyrightStatus]
        if isinstance(_copyright_status, Unset):
            copyright_status = UNSET
        else:
            copyright_status = ContentItemCopyrightStatus(_copyright_status)

        type = d.pop("type", UNSET)

        _source_medium = d.pop("sourceMedium", UNSET)
        source_medium: Union[Unset, ContentItemSourceMedium]
        if isinstance(_source_medium, Unset):
            source_medium = UNSET
        else:
            source_medium = ContentItemSourceMedium(_source_medium)

        title = d.pop("title", UNSET)

        transcript = d.pop("transcript", UNSET)

        _entities = d.pop("entities", UNSET)
        entities: Union[Unset, ContentItemNamedEntitiesInformation]
        if isinstance(_entities, Unset):
            entities = UNSET
        else:
            entities = ContentItemNamedEntitiesInformation.from_dict(_entities)

        _mentions = d.pop("mentions", UNSET)
        mentions: Union[Unset, ContentItemEntitiesMentionsInformation]
        if isinstance(_mentions, Unset):
            mentions = UNSET
        else:
            mentions = ContentItemEntitiesMentionsInformation.from_dict(_mentions)

        topics = []
        _topics = d.pop("topics", UNSET)
        for topics_item_data in _topics or []:
            topics_item = TopicMention.from_dict(topics_item_data)

            topics.append(topics_item)

        embeddings = cast(List[str], d.pop("embeddings", UNSET))

        transcript_length = d.pop("transcriptLength", UNSET)

        total_pages = d.pop("totalPages", UNSET)

        language_code = d.pop("languageCode", UNSET)

        is_on_front_page = d.pop("isOnFrontPage", UNSET)

        _publication_date = d.pop("publicationDate", UNSET)
        publication_date: Union[Unset, datetime.datetime]
        if isinstance(_publication_date, Unset):
            publication_date = UNSET
        else:
            publication_date = isoparse(_publication_date)

        issue_uid = d.pop("issueUid", UNSET)

        country_code = d.pop("countryCode", UNSET)

        provider_code = d.pop("providerCode", UNSET)

        media_uid = d.pop("mediaUid", UNSET)

        _media_type = d.pop("mediaType", UNSET)
        media_type: Union[Unset, ContentItemMediaType]
        if isinstance(_media_type, Unset):
            media_type = UNSET
        else:
            media_type = ContentItemMediaType(_media_type)

        has_olr = d.pop("hasOLR", UNSET)

        ocr_quality_score = d.pop("ocrQualityScore", UNSET)

        relevance_score = d.pop("relevanceScore", UNSET)

        page_numbers = cast(List[float], d.pop("pageNumbers", UNSET))

        collection_uids = cast(List[str], d.pop("collectionUids", UNSET))

        content_item = cls(
            uid=uid,
            copyright_status=copyright_status,
            type=type,
            source_medium=source_medium,
            title=title,
            transcript=transcript,
            entities=entities,
            mentions=mentions,
            topics=topics,
            embeddings=embeddings,
            transcript_length=transcript_length,
            total_pages=total_pages,
            language_code=language_code,
            is_on_front_page=is_on_front_page,
            publication_date=publication_date,
            issue_uid=issue_uid,
            country_code=country_code,
            provider_code=provider_code,
            media_uid=media_uid,
            media_type=media_type,
            has_olr=has_olr,
            ocr_quality_score=ocr_quality_score,
            relevance_score=relevance_score,
            page_numbers=page_numbers,
            collection_uids=collection_uids,
        )

        return content_item



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="ImageImageTypes")


@_attrs_define
class ImageImageTypes:
    """
    Attributes:
        visual_content (Union[Unset, str]): Whether the content is an image or not.
        technique (Union[Unset, str]): Determines if the image is a photograph.
        communication_goal (Union[Unset, str]): Purpose or communicative function of the image.
        visual_content_type (Union[Unset, str]): Classification of the visual content.
    """

    visual_content: Union[Unset, str] = UNSET
    technique: Union[Unset, str] = UNSET
    communication_goal: Union[Unset, str] = UNSET
    visual_content_type: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        visual_content = self.visual_content

        technique = self.technique

        communication_goal = self.communication_goal

        visual_content_type = self.visual_content_type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if visual_content is not UNSET:
            field_dict["visualContent"] = visual_content
        if technique is not UNSET:
            field_dict["technique"] = technique
        if communication_goal is not UNSET:
            field_dict["communicationGoal"] = communication_goal
        if visual_content_type is not UNSET:
            field_dict["visualContentType"] = visual_content_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        visual_content = d.pop("visualContent", UNSET)

        technique = d.pop("technique", UNSET)

        communication_goal = d.pop("communicationGoal", UNSET)

        visual_content_type = d.pop("visualContentType", UNSET)

        image_image_types = cls(
            visual_content=visual_content,
            technique=technique,
            communication_goal=communication_goal,
            visual_content_type=visual_content_type,
        )

        return image_image_types



from impresso.api_client.client import AuthenticatedClient
from impresso.resources.text_reuse.clusters import TextReuseClustersResource
from impresso.resources.text_reuse.passages import TextReusePassagesResource


class TextReuseDomain:
    """Container for text reuse resources."""

    def __init__(self, api_client: AuthenticatedClient) -> None:
        self._api_client = api_client

    @property
    def clusters(self) -> TextReuseClustersResource:
        return TextReuseClustersResource(self._api_client)

    @property
    def passages(self) -> TextReusePassagesResource:
        return TextReusePassagesResource(self._api_client)



from enum import Enum
from typing import TypeVar

from impresso.api_client.types import Unset

V = TypeVar("V", bound=str | int)
E = TypeVar("E", bound=Enum)


def get_enum_from_literal(value: V | Unset, enum: type[E]) -> E | Unset:
    if isinstance(value, Unset):
        return value

    for member in enum:
        if member.value == value:
            return member
    raise ValueError(f"{value} is not a valid member of {enum}")


def get_enum_from_literal_required(value: V | Unset, enum: type[E]) -> E:
    if isinstance(value, Unset):
        raise ValueError(f"{value} is required")
        return value

    for member in enum:
        if member.value == value:
            return member
    raise ValueError(f"{value} is not a valid member of {enum}")



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.entity_details_type import EntityDetailsType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.wikidata_location import WikidataLocation
    from ..models.wikidata_person import WikidataPerson


T = TypeVar("T", bound="EntityDetails")


@_attrs_define
class EntityDetails:
    """An entity: location or person.

    Attributes:
        uid (str): Unique identifier of the entity
        label (Union[Unset, str]): Entity label
        type (Union[Unset, EntityDetailsType]):
        wikidata_id (Union[Unset, str]): Wikidata identifier of the entity.
        total_mentions (Union[Unset, int]): Total number of mentions of the entity.
        total_content_items (Union[Unset, int]): Total number of content items the entity is mentioned in.
        wikidata_details (Union['WikidataLocation', 'WikidataPerson', Unset]):
    """

    uid: str
    label: Union[Unset, str] = UNSET
    type: Union[Unset, EntityDetailsType] = UNSET
    wikidata_id: Union[Unset, str] = UNSET
    total_mentions: Union[Unset, int] = UNSET
    total_content_items: Union[Unset, int] = UNSET
    wikidata_details: Union["WikidataLocation", "WikidataPerson", Unset] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        from ..models.wikidata_person import WikidataPerson

        uid = self.uid

        label = self.label

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        wikidata_id = self.wikidata_id

        total_mentions = self.total_mentions

        total_content_items = self.total_content_items

        wikidata_details: Union[Dict[str, Any], Unset]
        if isinstance(self.wikidata_details, Unset):
            wikidata_details = UNSET
        elif isinstance(self.wikidata_details, WikidataPerson):
            wikidata_details = self.wikidata_details.to_dict()
        else:
            wikidata_details = self.wikidata_details.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
            }
        )
        if label is not UNSET:
            field_dict["label"] = label
        if type is not UNSET:
            field_dict["type"] = type
        if wikidata_id is not UNSET:
            field_dict["wikidataId"] = wikidata_id
        if total_mentions is not UNSET:
            field_dict["totalMentions"] = total_mentions
        if total_content_items is not UNSET:
            field_dict["totalContentItems"] = total_content_items
        if wikidata_details is not UNSET:
            field_dict["wikidataDetails"] = wikidata_details

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.wikidata_location import WikidataLocation
        from ..models.wikidata_person import WikidataPerson

        d = src_dict.copy()
        uid = d.pop("uid")

        label = d.pop("label", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, EntityDetailsType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = EntityDetailsType(_type)

        wikidata_id = d.pop("wikidataId", UNSET)

        total_mentions = d.pop("totalMentions", UNSET)

        total_content_items = d.pop("totalContentItems", UNSET)

        def _parse_wikidata_details(data: object) -> Union["WikidataLocation", "WikidataPerson", Unset]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                wikidata_details_type_0 = WikidataPerson.from_dict(data)

                return wikidata_details_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            wikidata_details_type_1 = WikidataLocation.from_dict(data)

            return wikidata_details_type_1

        wikidata_details = _parse_wikidata_details(d.pop("wikidataDetails", UNSET))

        entity_details = cls(
            uid=uid,
            label=label,
            type=type,
            wikidata_id=wikidata_id,
            total_mentions=total_mentions,
            total_content_items=total_content_items,
            wikidata_details=wikidata_details,
        )

        return entity_details



from enum import Enum
from typing import Literal


class FilterOp(str, Enum):
    AND = "AND"
    OR = "OR"

    def __str__(self) -> str:
        return str(self.value)


FilterOpLiteral = Literal[
    "AND",
    "OR",
]



from typing import Any, Dict, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="SearchFacetBucket")


@_attrs_define
class SearchFacetBucket:
    """Facet bucket

    Attributes:
        count (int): Number of items in the bucket
        value (Union[float, int, str]): Value that represents the bucket.
        label (Union[Unset, str]): Label of the value, if relevant.
    """

    count: int
    value: Union[float, int, str]
    label: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        count = self.count

        value: Union[float, int, str]
        value = self.value

        label = self.label

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "count": count,
                "value": value,
            }
        )
        if label is not UNSET:
            field_dict["label"] = label

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        count = d.pop("count")

        def _parse_value(data: object) -> Union[float, int, str]:
            return cast(Union[float, int, str], data)

        value = _parse_value(d.pop("value"))

        label = d.pop("label", UNSET)

        search_facet_bucket = cls(
            count=count,
            value=value,
            label=label,
        )

        return search_facet_bucket



from enum import Enum
from typing import Literal


class FindImagesOrderBy(str, Enum):
    DATE = "date"
    VALUE_1 = "-date"

    def __str__(self) -> str:
        return str(self.value)


FindImagesOrderByLiteral = Literal[
    "date",
    "-date",
]



import datetime
from collections.abc import Sequence as ABCSequence
from typing import (
    Any,
    Generic,
    Iterable,
    Literal,
    Sequence,
    TypeGuard,
    TypeVar,
    Union,
    cast,
)

T = TypeVar("T")


def _is_string_like_sequence(obj: Any) -> TypeGuard[Sequence[str]]:
    return (
        isinstance(obj, ABCSequence)
        and not isinstance(obj, str)
        and all(isinstance(item, str) for item in obj)
    )


class TermSet(set[T], Generic[T]):

    def __init__(self, items: Union[Sequence[T], T], *args: T):
        if _is_string_like_sequence(items):
            _items = items
        elif isinstance(items, str):
            _items = [items] + list(args)  # type:ignore
        else:
            raise ValueError(f"{items.__class__} is not supported")

        super().__init__(cast(Iterable[T], _items))
        self.inverted = False
        self.chain: list[TermSet] = []
        self._precision = "exact"

    def __invert__(self):
        new_instance = AND(list(self))
        new_instance.inverted = not self.inverted
        new_instance.chain = self.chain
        return new_instance

    def __and__(self, other: Any) -> "TermSet[T]":
        if not isinstance(other, TermSet):
            raise ValueError(f"{other.__class__} is not supported")
        klass = other.__class__
        new_instance = klass(list(other))
        new_instance.chain = other.chain + [self]
        new_instance.chain.extend(self.chain)
        new_instance.inverted = other.inverted
        return new_instance

    @property
    def op(self) -> Literal["AND", "OR"]:
        return getattr(self, "_op")

    @property
    def precision(self) -> Literal["fuzzy", "soft", "exact", "partial"]:
        return getattr(self, "_precision", "exact")


class AND(TermSet[T], Generic[T]):
    """
    Used in filters to specify that all the terms must be present in the result.

    Example:

    ```
    AND(["apple", "banana"])
    AND("apple")
    AND("apple", "banana")
    # Negate:
    ~AND("apple", "banana")
    ```

    """

    def __init__(self, items: Union[Sequence[T], T], *args: T):
        super().__init__(items, *args)
        self._op = "AND"


class OR(TermSet[T], Generic[T]):
    """
    Used in filters to specify that any of the terms must be present in the result.


    Example:

    ```
    OR(["apple", "banana"])
    OR("apple")
    OR("apple", "banana")
    # Negate:
    ~OR("apple", "banana")
    ```

    """

    def __init__(self, items: Union[Sequence[T], T], *args: T):
        super().__init__(items, *args)
        self._op = "OR"


def _as_term_set(val: TermSet[T] | str) -> TermSet[T]:
    if isinstance(val, str):
        return AND(cast(T, val))
    return val


def Fuzzy(ts: TermSet[T] | str) -> TermSet[T]:
    s = _as_term_set(ts)
    s._precision = "fuzzy"
    return s


def Soft(ts: TermSet[T] | str) -> TermSet[T]:
    s = _as_term_set(ts)
    s._precision = "soft"
    return s


def Exact(ts: TermSet[T] | str) -> TermSet[T]:
    s = _as_term_set(ts)
    s._precision = "exact"
    return s


def Partial(ts: TermSet[T] | str) -> TermSet[T]:
    s = _as_term_set(ts)
    s._precision = "partial"
    return s


class DateRange:
    """
    Date range.

    Example:

    ```
    DateRange(datetime.date(1900, 1, 1), datetime.date(2000, 12, 31))

    # Everything until 2000
    DateRange(None, datetime.date(2000, 12, 31))

    # Everything since 1900
    DateRange(datetime.date(1900, 12, 31), None)

    ```

    """

    def __init__(
        self, start: datetime.date | str | None, end: datetime.date | str | None
    ):
        self.start = datetime.date.min if start is None else DateRange._as_date(start)
        self.end = datetime.date.max if end is None else DateRange._as_date(end)
        self.inverted = False

    def __invert__(self):
        new_instance = DateRange(self.start, self.end)
        new_instance.inverted = not self.inverted
        return new_instance

    @staticmethod
    def _as_filter_value(v: "DateRange") -> str:
        return f"{v.start.isoformat()}T00:00:00Z TO {v.end.isoformat()}T00:00:00Z"

    @staticmethod
    def _as_date(value: datetime.date | str) -> datetime.date:
        if isinstance(value, str):
            return datetime.date.fromisoformat(value)
        return value


class NumericRange:
    """
    Numeric range.

    Example:

    ```
    NumericRange(1, 10)
    ```

    """

    def __init__(self, start: int, end: int):
        self.start = start
        self.end = end
        self.inverted = False

    def __invert__(self):
        new_instance = NumericRange(self.start, self.end)
        new_instance.inverted = not self.inverted
        return new_instance

    @staticmethod
    def _as_filter_value(v: "NumericRange") -> str:
        return f"{v.start} TO {v.end}"



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.topic_word import TopicWord


T = TypeVar("T", bound="Topic")


@_attrs_define
class Topic:
    """A topic

    Attributes:
        uid (str): The unique identifier of the topic
        language (str): The language code of the topic
        content_items_count (Union[Unset, float]): Number of content items with this topic
        words (Union[Unset, List['TopicWord']]): Top N words associated with the topic
        model (Union[Unset, str]): ID of the model used to generate the topic
    """

    uid: str
    language: str
    content_items_count: Union[Unset, float] = UNSET
    words: Union[Unset, List["TopicWord"]] = UNSET
    model: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        language = self.language

        content_items_count = self.content_items_count

        words: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.words, Unset):
            words = []
            for words_item_data in self.words:
                words_item = words_item_data.to_dict()
                words.append(words_item)

        model = self.model

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
                "language": language,
            }
        )
        if content_items_count is not UNSET:
            field_dict["contentItemsCount"] = content_items_count
        if words is not UNSET:
            field_dict["words"] = words
        if model is not UNSET:
            field_dict["model"] = model

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.topic_word import TopicWord

        d = src_dict.copy()
        uid = d.pop("uid")

        language = d.pop("language")

        content_items_count = d.pop("contentItemsCount", UNSET)

        words = []
        _words = d.pop("words", UNSET)
        for words_item_data in _words or []:
            words_item = TopicWord.from_dict(words_item_data)

            words.append(words_item)

        model = d.pop("model", UNSET)

        topic = cls(
            uid=uid,
            language=language,
            content_items_count=content_items_count,
            words=words,
            model=model,
        )

        return topic



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.entity_mention import EntityMention


T = TypeVar("T", bound="ContentItemEntitiesMentionsInformation")


@_attrs_define
class ContentItemEntitiesMentionsInformation:
    """A collection of entity mentions (location, person, etc.) present in text.

    Attributes:
        locations (Union[Unset, List['EntityMention']]): Locations mentioned in the content item.
        persons (Union[Unset, List['EntityMention']]): Persons mentioned in the content item.
        organisations (Union[Unset, List['EntityMention']]): Organisations mentioned in the content item.
        news_agencies (Union[Unset, List['EntityMention']]): News agencies mentioned in the content item.
    """

    locations: Union[Unset, List["EntityMention"]] = UNSET
    persons: Union[Unset, List["EntityMention"]] = UNSET
    organisations: Union[Unset, List["EntityMention"]] = UNSET
    news_agencies: Union[Unset, List["EntityMention"]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        locations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.locations, Unset):
            locations = []
            for locations_item_data in self.locations:
                locations_item = locations_item_data.to_dict()
                locations.append(locations_item)

        persons: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.persons, Unset):
            persons = []
            for persons_item_data in self.persons:
                persons_item = persons_item_data.to_dict()
                persons.append(persons_item)

        organisations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.organisations, Unset):
            organisations = []
            for organisations_item_data in self.organisations:
                organisations_item = organisations_item_data.to_dict()
                organisations.append(organisations_item)

        news_agencies: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.news_agencies, Unset):
            news_agencies = []
            for news_agencies_item_data in self.news_agencies:
                news_agencies_item = news_agencies_item_data.to_dict()
                news_agencies.append(news_agencies_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if locations is not UNSET:
            field_dict["locations"] = locations
        if persons is not UNSET:
            field_dict["persons"] = persons
        if organisations is not UNSET:
            field_dict["organisations"] = organisations
        if news_agencies is not UNSET:
            field_dict["newsAgencies"] = news_agencies

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.entity_mention import EntityMention

        d = src_dict.copy()
        locations = []
        _locations = d.pop("locations", UNSET)
        for locations_item_data in _locations or []:
            locations_item = EntityMention.from_dict(locations_item_data)

            locations.append(locations_item)

        persons = []
        _persons = d.pop("persons", UNSET)
        for persons_item_data in _persons or []:
            persons_item = EntityMention.from_dict(persons_item_data)

            persons.append(persons_item)

        organisations = []
        _organisations = d.pop("organisations", UNSET)
        for organisations_item_data in _organisations or []:
            organisations_item = EntityMention.from_dict(organisations_item_data)

            organisations.append(organisations_item)

        news_agencies = []
        _news_agencies = d.pop("newsAgencies", UNSET)
        for news_agencies_item_data in _news_agencies or []:
            news_agencies_item = EntityMention.from_dict(news_agencies_item_data)

            news_agencies.append(news_agencies_item)

        content_item_entities_mentions_information = cls(
            locations=locations,
            persons=persons,
            organisations=organisations,
            news_agencies=news_agencies,
        )

        return content_item_entities_mentions_information



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="ImpressoNamedEntityRecognitionEntityWikidata")


@_attrs_define
class ImpressoNamedEntityRecognitionEntityWikidata:
    """
    Attributes:
        id (str): Wikidata ID of the entity
        wikipedia_page_name (Union[Unset, str]): Wikipedia page name of the entity
        wikipedia_page_url (Union[Unset, str]): Wikipedia page URL of the entity
    """

    id: str
    wikipedia_page_name: Union[Unset, str] = UNSET
    wikipedia_page_url: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        wikipedia_page_name = self.wikipedia_page_name

        wikipedia_page_url = self.wikipedia_page_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
            }
        )
        if wikipedia_page_name is not UNSET:
            field_dict["wikipediaPageName"] = wikipedia_page_name
        if wikipedia_page_url is not UNSET:
            field_dict["wikipediaPageUrl"] = wikipedia_page_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        wikipedia_page_name = d.pop("wikipediaPageName", UNSET)

        wikipedia_page_url = d.pop("wikipediaPageUrl", UNSET)

        impresso_named_entity_recognition_entity_wikidata = cls(
            id=id,
            wikipedia_page_name=wikipedia_page_name,
            wikipedia_page_url=wikipedia_page_url,
        )

        return impresso_named_entity_recognition_entity_wikidata



from enum import Enum
from typing import Literal


class WikidataLocationType(str, Enum):
    LOCATION = "location"

    def __str__(self) -> str:
        return str(self.value)


WikidataLocationTypeLiteral = Literal["location",]



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, cast

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.base_find_response_pagination import BaseFindResponsePagination


T = TypeVar("T", bound="BaseFindResponse")


@_attrs_define
class BaseFindResponse:
    """
    Attributes:
        data (List[Any]):
        pagination (BaseFindResponsePagination):
    """

    data: List[Any]
    pagination: "BaseFindResponsePagination"

    def to_dict(self) -> Dict[str, Any]:
        data = self.data

        pagination = self.pagination.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "data": data,
                "pagination": pagination,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.base_find_response_pagination import BaseFindResponsePagination

        d = src_dict.copy()
        data = cast(List[Any], d.pop("data"))

        pagination = BaseFindResponsePagination.from_dict(d.pop("pagination"))

        base_find_response = cls(
            data=data,
            pagination=pagination,
        )

        return base_find_response



from enum import Enum
from typing import Literal


class CollectionAccessLevel(str, Enum):
    PRIVATE = "private"
    PUBLIC = "public"

    def __str__(self) -> str:
        return str(self.value)


CollectionAccessLevelLiteral = Literal[
    "private",
    "public",
]



from enum import Enum
from typing import Literal


class GetTrClustersFacetId(str, Enum):
    DATERANGE = "daterange"
    NEWSPAPER = "newspaper"
    TEXTREUSECLUSTERDAYDELTA = "textReuseClusterDayDelta"
    TEXTREUSECLUSTERLEXICALOVERLAP = "textReuseClusterLexicalOverlap"
    TEXTREUSECLUSTERSIZE = "textReuseClusterSize"

    def __str__(self) -> str:
        return str(self.value)


GetTrClustersFacetIdLiteral = Literal[
    "daterange",
    "newspaper",
    "textReuseClusterDayDelta",
    "textReuseClusterLexicalOverlap",
    "textReuseClusterSize",
]



from typing import Any, Callable, Iterator, cast
from pandas import DataFrame, json_normalize
from impresso.api_client.api.collections import (
    find_collections,
    get_collection,
    patch_collections_collection_id_items,
)
from impresso.api_client.models.find_collections_base_find_response import (
    FindCollectionsBaseFindResponse,
)
from impresso.api_client.models.find_collections_order_by import (
    FindCollectionsOrderBy,
    FindCollectionsOrderByLiteral,
)
from impresso.api_client.models.update_collectable_items_request import (
    UpdateCollectableItemsRequest,
)
from impresso.api_client.types import UNSET
from impresso.api_models import BaseFind, Collection
from impresso.data_container import DataContainer, iterate_pages
from impresso.resources.base import Resource
from impresso.resources.search import SearchDataContainer, SearchResource
from impresso.util.error import raise_for_error
from impresso.util.py import get_enum_from_literal


class FindCollectionsSchema(BaseFind):
    """Schema for the find collections response."""

    data: list[Collection]


class FindCollectionsContainer(DataContainer):
    """Response of a find call."""

    def __init__(
        self,
        data: FindCollectionsBaseFindResponse,
        pydantic_model: type[FindCollectionsSchema],
        fetch_method: Callable[..., "FindCollectionsContainer"],
        fetch_method_args: dict[str, Any],
        web_app_search_result_url: str | None = None,
    ):
        super().__init__(data, pydantic_model, web_app_search_result_url)
        self._fetch_method = fetch_method
        self._fetch_method_args = fetch_method_args

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            return json_normalize(self._data.to_dict()["data"]).set_index("uid")
        return DataFrame()

    def pages(self) -> Iterator["FindCollectionsContainer"]:
        yield self
        yield from iterate_pages(
            self._fetch_method,
            self._fetch_method_args,
            self.offset,
            self.limit,
            self.total,
        )


class GetCollectionContainer(DataContainer):
    """Response of a get call."""

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()
        if len(data):
            return json_normalize([self._data.to_dict()]).set_index("uid")
        return DataFrame()

    @property
    def size(self) -> int:
        """Current page size."""
        data = self._data.to_dict()
        if len(data):
            return 1
        return 0

    @property
    def total(self) -> int:
        """Total number of results."""
        return self.size


class CollectionsResource(Resource):
    """
    Work with collections.

    Examples:
        Find collections containing the term "war":
        >>> results = collections.find(term="war") # doctest: +SKIP
        >>> print(results.df) # doctest: +SKIP

        Get a specific collection by its ID:
        >>> collection_id = "some-collection-id" # Replace with a real ID
        >>> collection = collections.get(collection_id) # doctest: +SKIP
        >>> print(collection.df) # doctest: +SKIP

        List items in a collection:
        >>> items = collections.items(collection_id) # doctest: +SKIP
        >>> print(items.df) # doctest: +SKIP

        Add items to a collection:
        >>> item_ids_to_add = ["item-id-1", "item-id-2"] # Replace with real item IDs
        >>> collections.add_items(collection_id, item_ids_to_add) # doctest: +SKIP

        Remove items from a collection:
        >>> item_ids_to_remove = ["item-id-1"] # Replace with real item IDs
        >>> collections.remove_items(collection_id, item_ids_to_remove) # doctest: +SKIP
    """

    name = "collections"

    def find(
        self,
        term: str | None = None,
        order_by: FindCollectionsOrderByLiteral | None = None,
        limit: int | None = None,
        offset: int | None = None,
    ) -> FindCollectionsContainer:
        """
        Search collections in Impresso.

        Args:
            term: Search term.
            order_by: Order by aspect.
            limit: Number of results to return.
            offset: Number of results to skip.

        Returns:
            FindCollectionsContainer: Data container with a page of results of the search.
        """

        result = find_collections.sync(
            client=self._api_client,
            term=term if term is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, FindCollectionsOrderBy)  # type: ignore
                if order_by is not None
                else FindCollectionsOrderBy.VALUE_0
            ),
            limit=limit if limit is not None else UNSET,
            offset=offset if offset is not None else UNSET,
        )
        raise_for_error(result)
        return FindCollectionsContainer(
            # assuming it's not an error because we raised for error above
            cast(FindCollectionsBaseFindResponse, result),
            FindCollectionsSchema,
            fetch_method=self.find,
            fetch_method_args={
                "term": term,
                "order_by": order_by,
            },
            web_app_search_result_url=_build_web_app_find_collections_url(
                base_url=self._get_web_app_base_url(),
                term=term,
                order_by=order_by,
            ),
        )

    def get(self, id: str) -> GetCollectionContainer:
        """Get collection by ID.

        Args:
            id: The ID of the collection to retrieve.

        Returns:
            GetCollectionContainer: Data container with the collection information.
        """

        result = get_collection.sync(
            client=self._api_client,
            id=id,
        )
        raise_for_error(result)
        return GetCollectionContainer(
            result,
            FindCollectionsSchema,
            web_app_search_result_url=_build_web_app_get_collection_url(
                base_url=self._get_web_app_base_url(),
                collection_id=id,
            ),
        )

    def items(
        self,
        collection_id: str,
        limit: int | None = None,
        offset: int | None = None,
    ) -> SearchDataContainer:
        """
        Return all content items from a collection.

        Args:
            collection_id: ID of the collection.
            limit: Number of results to return.
            offset: Number of results to skip.

        Returns:
            SearchDataContainer: Data container with a page of results of the search.
        """

        search_resource = SearchResource(self._api_client)
        return search_resource.find(
            collection_id=collection_id, limit=limit, offset=offset
        )

    def add_items(self, collection_id: str, item_ids: list[str]) -> None:
        """
        Add items to a collection by their IDs.

        **NOTE**: Items are not added immediately.
        This operation may take up to a few minutes
        to complete and reflect in the collection.

        Args:
            collection_id: ID of the collection.
            item_ids: IDs of the content items to add.
        """
        result = patch_collections_collection_id_items.sync(
            client=self._api_client,
            collection_id=collection_id,
            body=UpdateCollectableItemsRequest(
                add=item_ids,
                remove=UNSET,
            ),
        )
        raise_for_error(result)

    def remove_items(self, collection_id: str, item_ids: list[str]) -> None:
        """
        Remove items from a collection by their IDs.

        **NOTE**: Items are not removed immediately.
        This operation may take up to a few minutes
        to complete and reflect in the collection.

        Args:
            collection_id: ID of the collection.
            item_ids: IDs of the content items to remove.
        """
        result = patch_collections_collection_id_items.sync(
            client=self._api_client,
            collection_id=collection_id,
            body=UpdateCollectableItemsRequest(
                remove=item_ids,
                add=UNSET,
            ),
        )
        raise_for_error(result)


def _build_web_app_find_collections_url(
    base_url: str,
    term: str | None = None,
    order_by: FindCollectionsOrderByLiteral | None = None,
) -> str:
    query_params = {
        "orderBy": order_by,
        "q": term,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    url = f"{base_url}/collections?{query_string}"
    return f"{url}?{query_string}" if query_string else url


def _build_web_app_get_collection_url(
    base_url: str,
    collection_id: str,
) -> str:
    return f"{base_url}/collections/{collection_id}"



from typing import Any, Callable, Iterator, cast

from pandas import DataFrame, json_normalize

from impresso.api_client.api.search_facets import get_tr_passages_facet
from impresso.api_client.api.text_reuse_passages import find_text_reuse_passages
from impresso.api_client.models.find_text_reuse_passages_order_by import (
    FindTextReusePassagesOrderBy,
    FindTextReusePassagesOrderByLiteral,
)
from impresso.api_client.models.get_tr_passages_facet_id import (
    GetTrPassagesFacetId,
    GetTrPassagesFacetIdLiteral,
)
from impresso.api_client.models.get_tr_passages_facet_order_by import (
    GetTrPassagesFacetOrderBy,
)
from impresso.api_client.types import UNSET, Unset
from impresso.api_client.models.find_text_reuse_passages_base_find_response import (
    FindTextReusePassagesBaseFindResponse,
)
from impresso.api_client.models.get_tr_passages_facet_base_find_response import (
    GetTrPassagesFacetBaseFindResponse,
)
from impresso.api_models import (
    BaseFind,
    SearchFacetBucket,
    TextReusePassage,
)
from impresso.data_container import DataContainer, iterate_pages
from impresso.resources.base import Resource
from impresso.resources.text_reuse.clusters import Range, _build_filters
from impresso.structures import AND, OR, DateRange
from impresso.util.error import raise_for_error
from impresso.util.filters import and_or_filter, filters_as_protobuf
from impresso.util.py import get_enum_from_literal


class FindTextReusePassageResponseSchema(BaseFind):
    """Schema for the text reuse passage response."""

    data: list[TextReusePassage]


class PassagesFacetResponseSchema(BaseFind):
    """Schema for the text reuse passages facet response."""

    data: list[SearchFacetBucket]


class FindTextReusePassagesContainer(DataContainer):
    """Response of a find text reuse passages call."""

    def __init__(
        self,
        data: FindTextReusePassagesBaseFindResponse,
        pydantic_model: type[FindTextReusePassageResponseSchema],
        fetch_method: Callable[..., "FindTextReusePassagesContainer"],
        fetch_method_args: dict[str, Any],
        web_app_search_result_url: str | None = None,
    ):
        super().__init__(data, pydantic_model, web_app_search_result_url)
        self._fetch_method = fetch_method
        self._fetch_method_args = fetch_method_args

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            return json_normalize(self._data.to_dict()["data"]).set_index("uid")
        return DataFrame()

    def pages(self) -> Iterator["FindTextReusePassagesContainer"]:
        """Iterate over all pages of results."""
        yield self
        yield from iterate_pages(
            self._fetch_method,
            self._fetch_method_args,
            self.offset,
            self.limit,
            self.total,
        )


class PassagesFacetDataContainer(DataContainer):
    """Response of a get text reuse passages facet call."""

    def __init__(
        self,
        data: GetTrPassagesFacetBaseFindResponse,
        pydantic_model: type[PassagesFacetResponseSchema],
        fetch_method: Callable[..., "PassagesFacetDataContainer"],
        fetch_method_args: dict[str, Any],
        web_app_search_result_url: str | None = None,
    ):
        super().__init__(data, pydantic_model, web_app_search_result_url)
        self._fetch_method = fetch_method
        self._fetch_method_args = fetch_method_args

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            return json_normalize(self._data.to_dict()["data"]).set_index("value")
        return DataFrame()

    def pages(self) -> Iterator["PassagesFacetDataContainer"]:
        """Iterate over all pages of results."""
        yield self
        yield from iterate_pages(
            self._fetch_method,
            self._fetch_method_args,
            self.offset,
            self.limit,
            self.total,
        )


class TextReusePassagesResource(Resource):
    """Text reuse passages resource."""

    name = "textReusePassages"

    def find(
        self,
        term: str | None = None,
        limit: int | None = None,
        offset: int | None = None,
        order_by: FindTextReusePassagesOrderByLiteral | None = None,
        cluster_id: str | AND[str] | OR[str] | None = None,
        cluster_size: Range | AND[Range] | OR[Range] | None = None,
        title: str | AND[str] | OR[str] | None = None,
        lexical_overlap: Range | AND[Range] | OR[Range] | None = None,
        day_delta: Range | AND[Range] | OR[Range] | None = None,
        date_range: DateRange | None = None,
        newspaper_id: str | OR[str] | None = None,
        collection_id: str | OR[str] | None = None,
        front_page: bool | None = None,
        topic_id: str | AND[str] | OR[str] | None = None,
        language: str | OR[str] | None = None,
        country: str | OR[str] | None = None,
        mention: str | AND[str] | OR[str] | None = None,
        entity_id: str | AND[str] | OR[str] | None = None,
    ) -> FindTextReusePassagesContainer:
        """
        Find text reuse passages based on various criteria.

        Args:
            term: Search for passages containing specific text.
            limit: Maximum number of passages to return.
            offset: Number of passages to skip from the beginning.
            order_by: Specify the sorting order for the results.
            cluster_id: Filter passages belonging to specific text reuse clusters.
            cluster_size: Filter passages based on the size of the cluster they belong to.
            title: Filter passages by the title of the articles they appear in.
            lexical_overlap: Filter passages based on the lexical overlap score within their cluster.
            day_delta: Filter passages based on the time span (in days) of their cluster.
            date_range: Filter passages based on their publication date.
            newspaper_id: Filter passages from specific newspapers.
            collection_id: Filter passages from specific collections.
            front_page: Filter passages appearing on the front page.
            topic_id: Filter passages associated with specific topics.
            language: Filter passages by their language.
            country: Filter passages by the country of publication.
            mention: Filter passages containing specific mentions (named entities).
            entity_id: Filter passages associated with specific entity IDs.

        Returns:
            FindTextReusePassagesContainer: A container holding the search results.

        Examples:
            Find passages containing the term 'revolution' from French newspapers:
            >>> results = textReusePassages.find(term='revolution', country='FR') # doctest: +SKIP
            >>> print(results.df) # doctest: +SKIP

            Find passages from clusters with a size greater than 50:
            >>> results = textReusePassages.find(cluster_size=(51, None)) # doctest: +SKIP
            >>> print(results.df) # doctest: +SKIP
        """
        # reusing build filters from clusters - they are the same
        filters = _build_filters(
            cluster_id=cluster_id,
            cluster_size=cluster_size,
            title=title,
            lexical_overlap=lexical_overlap,
            day_delta=day_delta,
            date_range=date_range,
            newspaper_id=newspaper_id,
            collection_id=collection_id,
            front_page=front_page,
            topic_id=topic_id,
            language=language,
            country=country,
            mention=mention,
            entity_id=entity_id,
        )
        if term is not None:
            filters.extend(and_or_filter(term, "string"))
        filters_pb = filters_as_protobuf(filters or [])

        result = find_text_reuse_passages.sync(
            client=self._api_client,
            limit=limit if limit is not None else UNSET,
            offset=offset if offset is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, FindTextReusePassagesOrderBy)
                if order_by is not None
                else UNSET
            ),
            filters=filters_pb if filters_pb else UNSET,
        )
        raise_for_error(result)
        return FindTextReusePassagesContainer(
            data=cast(FindTextReusePassagesBaseFindResponse, result),
            pydantic_model=FindTextReusePassageResponseSchema,
            fetch_method=self.find,
            fetch_method_args={
                "term": term,
                "order_by": order_by,
                "cluster_id": cluster_id,
                "cluster_size": cluster_size,
                "title": title,
                "lexical_overlap": lexical_overlap,
                "day_delta": day_delta,
                "date_range": date_range,
                "newspaper_id": newspaper_id,
                "collection_id": collection_id,
                "front_page": front_page,
                "topic_id": topic_id,
                "language": language,
                "country": country,
                "mention": mention,
                "entity_id": entity_id,
            },
            web_app_search_result_url=_build_web_app_find_passages_url(
                base_url=self._get_web_app_base_url(),
                filters=filters_pb,
                limit=limit,
                offset=offset,
                order_by=order_by,
            ),
        )

    def facet(
        self,
        facet: GetTrPassagesFacetIdLiteral,
        term: str | None = None,
        limit: int | None = None,
        offset: int | None = None,
        order_by: FindTextReusePassagesOrderByLiteral | None = None,
        cluster_id: str | AND[str] | OR[str] | None = None,
        cluster_size: Range | AND[Range] | OR[Range] | None = None,
        title: str | AND[str] | OR[str] | None = None,
        lexical_overlap: Range | AND[Range] | OR[Range] | None = None,
        day_delta: Range | AND[Range] | OR[Range] | None = None,
        date_range: DateRange | None = None,
        newspaper_id: str | OR[str] | None = None,
        collection_id: str | OR[str] | None = None,
        front_page: bool | None = None,
        topic_id: str | AND[str] | OR[str] | None = None,
        language: str | OR[str] | None = None,
        country: str | OR[str] | None = None,
        mention: str | AND[str] | OR[str] | None = None,
        entity_id: str | AND[str] | OR[str] | None = None,
    ) -> PassagesFacetDataContainer:
        """
        Get facet information for text reuse passages based on specified filters.

        Facets provide aggregated counts for different properties of the passages,
        such as the distribution of newspapers or languages.

        Args:
            facet: The specific facet to retrieve (e.g., 'newspaper', 'language').
            term: Filter passages by text content before calculating facets.
            limit: Maximum number of facet values to return.
            offset: Number of facet values to skip.
            order_by: How to order the facet values (e.g., 'value', 'count').
            cluster_id: Filter passages by cluster ID before calculating facets.
            cluster_size: Filter passages by cluster size before calculating facets.
            title: Filter passages by article title before calculating facets.
            lexical_overlap: Filter passages by lexical overlap before calculating facets.
            day_delta: Filter passages by cluster day delta before calculating facets.
            date_range: Filter passages by publication date before calculating facets.
            newspaper_id: Filter passages by newspaper before calculating facets.
            collection_id: Filter passages by collection before calculating facets.
            front_page: Filter passages by front page status before calculating facets.
            topic_id: Filter passages by topic ID before calculating facets.
            language: Filter passages by language before calculating facets.
            country: Filter passages by country before calculating facets.
            mention: Filter passages by mention before calculating facets.
            entity_id: Filter passages by entity ID before calculating facets.

        Returns:
            FacetDataContainer: A container holding the facet results.

        Examples:
            Get the top 10 newspapers associated with passages containing 'war':
            >>> facet_results = textReusePassages.facet(facet='newspaper', term='war', limit=10) # doctest: +SKIP
            >>> print(facet_results.df) # doctest: +SKIP

            Get the language distribution for passages published between 1914 and 1918:
            >>> from impresso.structures import DateRange
            >>> date_filter = DateRange(start="1914-01-01", end="1918-12-31")
            >>> facet_results = textReusePassages.facet(facet='language', date_range=date_filter) # doctest: +SKIP
            >>> print(facet_results.df) # doctest: +SKIP
        """
        facet_id = get_enum_from_literal(facet, GetTrPassagesFacetId)
        if isinstance(facet_id, Unset):
            raise ValueError(f"{facet} is not a valid value")

        filters = _build_filters(
            cluster_id=cluster_id,
            cluster_size=cluster_size,
            title=title,
            lexical_overlap=lexical_overlap,
            day_delta=day_delta,
            date_range=date_range,
            newspaper_id=newspaper_id,
            collection_id=collection_id,
            front_page=front_page,
            topic_id=topic_id,
            language=language,
            country=country,
            mention=mention,
            entity_id=entity_id,
        )

        if term is not None:
            filters.extend(and_or_filter(term, "string"))

        filters_pb = filters_as_protobuf(filters or [])

        result = get_tr_passages_facet.sync(
            client=self._api_client,
            id=facet_id,
            filters=filters_pb if filters_pb else UNSET,
            offset=offset if offset is not None else UNSET,
            limit=limit if limit is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, GetTrPassagesFacetOrderBy)
                if order_by is not None
                else get_enum_from_literal("value", GetTrPassagesFacetOrderBy)
            ),
        )
        raise_for_error(result)
        return PassagesFacetDataContainer(
            data=cast(GetTrPassagesFacetBaseFindResponse, result),
            pydantic_model=PassagesFacetResponseSchema,
            fetch_method=self.facet,
            fetch_method_args={
                "facet": facet,
                "term": term,
                "order_by": order_by,
                "cluster_id": cluster_id,
                "cluster_size": cluster_size,
                "title": title,
                "lexical_overlap": lexical_overlap,
                "day_delta": day_delta,
                "date_range": date_range,
                "newspaper_id": newspaper_id,
                "collection_id": collection_id,
                "front_page": front_page,
                "topic_id": topic_id,
                "language": language,
                "country": country,
                "mention": mention,
                "entity_id": entity_id,
            },
            web_app_search_result_url=_build_web_app_find_passages_url(
                base_url=self._get_web_app_base_url(),
                filters=filters_pb,
                limit=limit,
                offset=offset,
                order_by=order_by,
            ),
        )


def _build_web_app_find_passages_url(
    base_url: str,
    filters=str | None,
    limit=int | None,
    offset=int | None,
    order_by=FindTextReusePassagesOrderBy | None,
) -> str:
    page = offset // limit if limit is not None and offset is not None else 0
    query_params = {
        "sort": order_by,
        "sq": filters,
        "p": page + 1,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    url = f"{base_url}/text-reuse/passages"
    return f"{url}?{query_string}" if query_string else url



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.error import Error
from ...models.filter_ import Filter
from ...models.find_topics_base_find_response import FindTopicsBaseFindResponse
from ...models.find_topics_order_by import FindTopicsOrderBy
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    q: Union[Unset, str] = UNSET,
    order_by: Union[Unset, FindTopicsOrderBy] = FindTopicsOrderBy.NAME,
    filters: Union[List["Filter"], Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["q"] = q

    json_order_by: Union[Unset, str] = UNSET
    if not isinstance(order_by, Unset):
        json_order_by = order_by.value

    params["order_by"] = json_order_by

    json_filters: Union[List[Dict[str, Any]], Unset, str]
    if isinstance(filters, Unset):
        json_filters = UNSET
    elif isinstance(filters, list):
        json_filters = []
        for filters_type_1_item_data in filters:
            filters_type_1_item = filters_type_1_item_data.to_dict()
            json_filters.append(filters_type_1_item)

    else:
        json_filters = filters
    params["filters"] = json_filters

    params["limit"] = limit

    params["offset"] = offset

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/topics",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Error, FindTopicsBaseFindResponse]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = FindTopicsBaseFindResponse.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = Error.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = Error.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = Error.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.IM_A_TEAPOT:
        response_418 = Error.from_dict(response.json())

        return response_418
    if response.status_code == HTTPStatus.UNPROCESSABLE_CONTENT:
        response_422 = Error.from_dict(response.json())

        return response_422
    if response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
        response_429 = Error.from_dict(response.json())

        return response_429
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = Error.from_dict(response.json())

        return response_500
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Error, FindTopicsBaseFindResponse]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    q: Union[Unset, str] = UNSET,
    order_by: Union[Unset, FindTopicsOrderBy] = FindTopicsOrderBy.NAME,
    filters: Union[List["Filter"], Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
) -> Response[Union[Error, FindTopicsBaseFindResponse]]:
    """Find topics

    Args:
        q (Union[Unset, str]):
        order_by (Union[Unset, FindTopicsOrderBy]):  Default: FindTopicsOrderBy.NAME.
        filters (Union[List['Filter'], Unset, str]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Error, FindTopicsBaseFindResponse]]
    """

    kwargs = _get_kwargs(
        q=q,
        order_by=order_by,
        filters=filters,
        limit=limit,
        offset=offset,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    q: Union[Unset, str] = UNSET,
    order_by: Union[Unset, FindTopicsOrderBy] = FindTopicsOrderBy.NAME,
    filters: Union[List["Filter"], Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
) -> Optional[Union[Error, FindTopicsBaseFindResponse]]:
    """Find topics

    Args:
        q (Union[Unset, str]):
        order_by (Union[Unset, FindTopicsOrderBy]):  Default: FindTopicsOrderBy.NAME.
        filters (Union[List['Filter'], Unset, str]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Error, FindTopicsBaseFindResponse]
    """

    return sync_detailed(
        client=client,
        q=q,
        order_by=order_by,
        filters=filters,
        limit=limit,
        offset=offset,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    q: Union[Unset, str] = UNSET,
    order_by: Union[Unset, FindTopicsOrderBy] = FindTopicsOrderBy.NAME,
    filters: Union[List["Filter"], Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
) -> Response[Union[Error, FindTopicsBaseFindResponse]]:
    """Find topics

    Args:
        q (Union[Unset, str]):
        order_by (Union[Unset, FindTopicsOrderBy]):  Default: FindTopicsOrderBy.NAME.
        filters (Union[List['Filter'], Unset, str]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Error, FindTopicsBaseFindResponse]]
    """

    kwargs = _get_kwargs(
        q=q,
        order_by=order_by,
        filters=filters,
        limit=limit,
        offset=offset,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    q: Union[Unset, str] = UNSET,
    order_by: Union[Unset, FindTopicsOrderBy] = FindTopicsOrderBy.NAME,
    filters: Union[List["Filter"], Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
) -> Optional[Union[Error, FindTopicsBaseFindResponse]]:
    """Find topics

    Args:
        q (Union[Unset, str]):
        order_by (Union[Unset, FindTopicsOrderBy]):  Default: FindTopicsOrderBy.NAME.
        filters (Union[List['Filter'], Unset, str]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Error, FindTopicsBaseFindResponse]
    """

    return (
        await asyncio_detailed(
            client=client,
            q=q,
            order_by=order_by,
            filters=filters,
            limit=limit,
            offset=offset,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.experiment_info import ExperimentInfo
    from ..models.find_experiments_base_find_response_pagination import FindExperimentsBaseFindResponsePagination


T = TypeVar("T", bound="FindExperimentsBaseFindResponse")


@_attrs_define
class FindExperimentsBaseFindResponse:
    """
    Attributes:
        data (List['ExperimentInfo']):
        pagination (FindExperimentsBaseFindResponsePagination):
    """

    data: List["ExperimentInfo"]
    pagination: "FindExperimentsBaseFindResponsePagination"

    def to_dict(self) -> Dict[str, Any]:
        data = []
        for data_item_data in self.data:
            data_item = data_item_data.to_dict()
            data.append(data_item)

        pagination = self.pagination.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "data": data,
                "pagination": pagination,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.experiment_info import ExperimentInfo
        from ..models.find_experiments_base_find_response_pagination import FindExperimentsBaseFindResponsePagination

        d = src_dict.copy()
        data = []
        _data = d.pop("data")
        for data_item_data in _data:
            data_item = ExperimentInfo.from_dict(data_item_data)

            data.append(data_item)

        pagination = FindExperimentsBaseFindResponsePagination.from_dict(d.pop("pagination"))

        find_experiments_base_find_response = cls(
            data=data,
            pagination=pagination,
        )

        return find_experiments_base_find_response



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.impresso_named_entity_recognition_request_method import ImpressoNamedEntityRecognitionRequestMethod
from ..types import UNSET, Unset

T = TypeVar("T", bound="ImpressoNamedEntityRecognitionRequest")


@_attrs_define
class ImpressoNamedEntityRecognitionRequest:
    """Request body for the Impresso NER endpoint

    Attributes:
        text (str): Text to be processed for named entity recognition
        method (Union[Unset, ImpressoNamedEntityRecognitionRequestMethod]): NER method to be used: `ner` (default),
            `ner-nel` (named entity recognition with named entity linking) and `nel` (linking only - enclose entities in
            [START] [END] tags). Default: ImpressoNamedEntityRecognitionRequestMethod.NER.
    """

    text: str
    method: Union[Unset, ImpressoNamedEntityRecognitionRequestMethod] = ImpressoNamedEntityRecognitionRequestMethod.NER

    def to_dict(self) -> Dict[str, Any]:
        text = self.text

        method: Union[Unset, str] = UNSET
        if not isinstance(self.method, Unset):
            method = self.method.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "text": text,
            }
        )
        if method is not UNSET:
            field_dict["method"] = method

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        text = d.pop("text")

        _method = d.pop("method", UNSET)
        method: Union[Unset, ImpressoNamedEntityRecognitionRequestMethod]
        if isinstance(_method, Unset):
            method = UNSET
        else:
            method = ImpressoNamedEntityRecognitionRequestMethod(_method)

        impresso_named_entity_recognition_request = cls(
            text=text,
            method=method,
        )

        return impresso_named_entity_recognition_request



from enum import Enum
from typing import Literal


class FindEntitiesOrderBy(str, Enum):
    COUNT = "count"
    COUNT_MENTIONS = "count-mentions"
    NAME = "name"
    RELEVANCE = "relevance"
    VALUE_1 = "-relevance"
    VALUE_3 = "-name"
    VALUE_5 = "-count"
    VALUE_7 = "-count-mentions"

    def __str__(self) -> str:
        return str(self.value)


FindEntitiesOrderByLiteral = Literal[
    "count",
    "count-mentions",
    "name",
    "relevance",
    "-relevance",
    "-name",
    "-count",
    "-count-mentions",
]



from enum import Enum
from typing import Literal


class RemoveCollectionResponseParamsStatus(str, Enum):
    DEL = "DEL"

    def __str__(self) -> str:
        return str(self.value)


RemoveCollectionResponseParamsStatusLiteral = Literal["DEL",]



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

from ..models.impresso_text_embedding_request_search_target import ImpressoTextEmbeddingRequestSearchTarget

T = TypeVar("T", bound="ImpressoTextEmbeddingRequest")


@_attrs_define
class ImpressoTextEmbeddingRequest:
    """Body of a request to the Impresso Text Embedding endpoint

    Attributes:
        search_target (ImpressoTextEmbeddingRequestSearchTarget): Which embedding space the embedding is going to be
            used in
        text (str): Text to be embedded
    """

    search_target: ImpressoTextEmbeddingRequestSearchTarget
    text: str

    def to_dict(self) -> Dict[str, Any]:
        search_target = self.search_target.value

        text = self.text

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "searchTarget": search_target,
                "text": text,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        search_target = ImpressoTextEmbeddingRequestSearchTarget(d.pop("searchTarget"))

        text = d.pop("text")

        impresso_text_embedding_request = cls(
            search_target=search_target,
            text=text,
        )

        return impresso_text_embedding_request



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.text_reuse_passage_offset import TextReusePassageOffset


T = TypeVar("T", bound="TextReusePassage")


@_attrs_define
class TextReusePassage:
    """Represents a passage of text that was identified as a part of a text reuse cluster

    Attributes:
        uid (str): Unique ID of the text reuse passage.
        content (Union[Unset, str]): Textual content of the passage.
        content_item_id (Union[Unset, str]): ID of the content item that the text reuse passage belongs to.
        offset (Union[Unset, TextReusePassageOffset]): Start and end offsets of the passage in the content item.
    """

    uid: str
    content: Union[Unset, str] = UNSET
    content_item_id: Union[Unset, str] = UNSET
    offset: Union[Unset, "TextReusePassageOffset"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        content = self.content

        content_item_id = self.content_item_id

        offset: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.offset, Unset):
            offset = self.offset.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
            }
        )
        if content is not UNSET:
            field_dict["content"] = content
        if content_item_id is not UNSET:
            field_dict["contentItemId"] = content_item_id
        if offset is not UNSET:
            field_dict["offset"] = offset

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.text_reuse_passage_offset import TextReusePassageOffset

        d = src_dict.copy()
        uid = d.pop("uid")

        content = d.pop("content", UNSET)

        content_item_id = d.pop("contentItemId", UNSET)

        _offset = d.pop("offset", UNSET)
        offset: Union[Unset, TextReusePassageOffset]
        if isinstance(_offset, Unset):
            offset = UNSET
        else:
            offset = TextReusePassageOffset.from_dict(_offset)

        text_reuse_passage = cls(
            uid=uid,
            content=content,
            content_item_id=content_item_id,
            offset=offset,
        )

        return text_reuse_passage



from enum import Enum
from typing import Literal


class FindTopicsOrderBy(str, Enum):
    MODEL = "model"
    NAME = "name"
    VALUE_1 = "-name"
    VALUE_3 = "-model"

    def __str__(self) -> str:
        return str(self.value)


FindTopicsOrderByLiteral = Literal[
    "model",
    "name",
    "-name",
    "-model",
]



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="Newspaper")


@_attrs_define
class Newspaper:
    """A newspaper

    Attributes:
        uid (str): The unique identifier of the newspaper.
        title (Union[Unset, str]): The title of the newspaper.
        start_year (Union[Unset, float]): The year of the first available article in the newspaper.
        end_year (Union[Unset, float]): The year of the last available article in the newspaper.
        language_codes (Union[Unset, List[str]]): ISO 639-1 codes of languages used in the newspaper.
        total_articles (Union[Unset, float]): Total number of articles in the newspaper.
        total_issues (Union[Unset, float]): Total number of issues in the newspaper.
        total_pages (Union[Unset, float]): Total number of pages in the newspaper.
    """

    uid: str
    title: Union[Unset, str] = UNSET
    start_year: Union[Unset, float] = UNSET
    end_year: Union[Unset, float] = UNSET
    language_codes: Union[Unset, List[str]] = UNSET
    total_articles: Union[Unset, float] = UNSET
    total_issues: Union[Unset, float] = UNSET
    total_pages: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        title = self.title

        start_year = self.start_year

        end_year = self.end_year

        language_codes: Union[Unset, List[str]] = UNSET
        if not isinstance(self.language_codes, Unset):
            language_codes = self.language_codes

        total_articles = self.total_articles

        total_issues = self.total_issues

        total_pages = self.total_pages

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
            }
        )
        if title is not UNSET:
            field_dict["title"] = title
        if start_year is not UNSET:
            field_dict["startYear"] = start_year
        if end_year is not UNSET:
            field_dict["endYear"] = end_year
        if language_codes is not UNSET:
            field_dict["languageCodes"] = language_codes
        if total_articles is not UNSET:
            field_dict["totalArticles"] = total_articles
        if total_issues is not UNSET:
            field_dict["totalIssues"] = total_issues
        if total_pages is not UNSET:
            field_dict["totalPages"] = total_pages

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        uid = d.pop("uid")

        title = d.pop("title", UNSET)

        start_year = d.pop("startYear", UNSET)

        end_year = d.pop("endYear", UNSET)

        language_codes = cast(List[str], d.pop("languageCodes", UNSET))

        total_articles = d.pop("totalArticles", UNSET)

        total_issues = d.pop("totalIssues", UNSET)

        total_pages = d.pop("totalPages", UNSET)

        newspaper = cls(
            uid=uid,
            title=title,
            start_year=start_year,
            end_year=end_year,
            language_codes=language_codes,
            total_articles=total_articles,
            total_issues=total_issues,
            total_pages=total_pages,
        )

        return newspaper



import os

from pydantic import BaseModel, Field, ValidationError
import yaml

from impresso.util.token import get_jwt_status

DEFAULT_API_URL = "https://impresso-project.ch/public-api/v1"


class ImpressoApiToken(BaseModel):
    url: str
    token: str


class ImpressoPyConfigContent(BaseModel):
    """Content of the configuration file."""

    tokens: list[ImpressoApiToken] = []
    default_api_url: str = Field(
        serialization_alias="defaultApiUrl",
        default=DEFAULT_API_URL,
    )


class ImpressoPyConfig:
    """File backed configuration of the library."""

    def __init__(self, config_file="~/.impresso_py.yml") -> None:
        self._config_file = config_file
        self._config = ImpressoPyConfigContent()

        filepath = os.path.expanduser(self._config_file)
        if os.path.exists(filepath):
            with open(filepath, "r", encoding="utf-8") as f:
                try:
                    config = yaml.safe_load(f)
                    self._config = ImpressoPyConfigContent.model_validate(config)
                except yaml.YAMLError as exc:
                    print("Error loading config file:", exc)
                except ValidationError as exc:
                    print("Error validating config file:", exc)

    @property
    def default_api_url(self) -> str:
        return self._config.default_api_url

    def get_token(self, url: str | None) -> str | None:
        """
        Return the token for the given API URL.
        Use the default API URL if no URL is provided.
        """
        the_url = url or self._config.default_api_url
        token = next((t.token for t in self._config.tokens if t.url == the_url), None)

        if token is None:
            return None
        token_status, _ = get_jwt_status(token)
        if token_status == "valid":
            return token

        return None

    def set_token(self, token: str, url: str | None) -> None:
        """
        Return the token for the given API URL.
        Use the default API URL if no URL is provided.
        """
        the_url = url or self._config.default_api_url
        token_container = next(
            (t for t in self._config.tokens if t.url == the_url), None
        )

        if token_container is None:
            self._config.tokens.append(ImpressoApiToken(url=the_url, token=token))
        else:
            token_container.token = token

        self._write_config()

    def _write_config(self) -> None:
        """Write the configuration to the file."""
        filepath = os.path.expanduser(self._config_file)
        with open(filepath, "w", encoding="utf-8") as f:
            yaml.dump(self._config.model_dump(exclude_none=True), f)



"""Impresso Python client library."""

import getpass
import importlib.metadata
import logging
import os
from urllib.parse import urlparse

import httpx

from impresso.api_client import AuthenticatedClient
from impresso.client_base import ImpressoApiResourcesBase
from impresso.config_file import DEFAULT_API_URL, ImpressoPyConfig
from impresso.util.token import get_jwt_status

logger = logging.getLogger(__name__)

version = importlib.metadata.version("impresso")


def _is_localhost_netloc(netloc: str) -> bool:
    return netloc.startswith("localhost") or netloc.startswith("127.0.0.1")


DEFAULT_LOCALHOST_TOKEN_NETLOC = "impresso-project.ch"


def _log_non_2xx(response: httpx.Response) -> None:
    if response.status_code >= 400:
        response.read()
        logging.error(
            f"Received error response ({response.status_code}): {response.text}"
        )


class ImpressoClient(ImpressoApiResourcesBase):
    """
    Client class for the impresso Python libary. This is the context for all
    interactions with the impresso API.
    """

    def __init__(self, api_url: str, api_bearer_token: str):
        self._api_url = api_url
        self._api_bearer_token = api_bearer_token
        super().__init__(
            AuthenticatedClient(
                base_url=self._api_url,
                token=self._api_bearer_token,
                headers={
                    "Accept": "application/json",
                    "User-Agent": f"impresso-py/${version}",
                },
                raise_on_unexpected_status=True,
                httpx_args={
                    "event_hooks": {
                        "response": [_log_non_2xx],
                    }
                },
                timeout=httpx.Timeout(
                    connect=20.0,
                    read=300.0,
                    write=10.0,
                    pool=5.0,
                ),
            )
        )

    @property
    def api_url(self) -> str:
        """
        Return the Impresso Public API URL currently in use.
        """
        return self._api_url


_PROMPT = """
Click on the following link to access the login page: {URL}
 - 🔤 Enter your email/password on this page.
 - 🔑 Once logged in, a secret token will be generated for you.
 - 📋 Copy this token and paste it into the input field below. Then press "Enter". 👇🏼.
"""


def connect(
    public_api_url: str | None = None,
    persisted_token: bool = True,
) -> ImpressoClient:
    """
    Connect to the Impresso API and return a client object.

    ```python
    from impresso import connect

    impresso = connect()
    ```

    Args:
        public_api_url (str): The URL of the Impresso API to connect to. By default using the default URL set
                              in the config file (~/.impresso_py.yml) or the Impresso default URL ({DEFAULT_API_URL}).
        persisted_token (bool): Whether to read and write token to the user directory
                                (~/.impresso_py.yml).
                                This is useful to avoid having to re-enter the token each time the
                                Jupiter notebook is restarted.
    """

    config = ImpressoPyConfig()

    api_url = public_api_url or os.getenv("IMPRESSO_API_URL") or config.default_api_url

    parsed_url = urlparse(api_url)
    token_base_url_netloc = parsed_url.netloc
    if _is_localhost_netloc(token_base_url_netloc):
        token_base_url_netloc = DEFAULT_LOCALHOST_TOKEN_NETLOC

    token_base_url = f"https://{token_base_url_netloc}"
    token_url = f"{token_base_url}/datalab/token"

    token = None
    if persisted_token:
        token = config.get_token(url=api_url)

    if not token:
        # Show a prompt to the user with the explanations on how to get the token.
        print(_PROMPT.format(URL=token_url))
        token = getpass.getpass("🔑 Enter your token: ")
        token_status, _ = get_jwt_status(token)

        if token_status != "valid":
            message = f"The provided token is {token_status}. Have you entered it correctly? 🤔"
            print(message)
            raise ValueError(message)

        if persisted_token:
            config.set_token(token, api_url)

    print("🎉 You are now connected to the Impresso API!  🎉")
    if api_url != DEFAULT_API_URL:
        print(f"🔗 Using API: {api_url}")

    return ImpressoClient(api_url=api_url, api_bearer_token=token)



import datetime
from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define
from dateutil.parser import isoparse

T = TypeVar("T", bound="TextReuseClusterTimeCoverage")


@_attrs_define
class TextReuseClusterTimeCoverage:
    """Time coverage of the cluster.

    Attributes:
        start_date (datetime.date): Publication date of the earliest content item in the cluster.
        end_date (datetime.date): Publication date of the latest content item in the cluster.
    """

    start_date: datetime.date
    end_date: datetime.date

    def to_dict(self) -> Dict[str, Any]:
        start_date = self.start_date.isoformat()

        end_date = self.end_date.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "startDate": start_date,
                "endDate": end_date,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        start_date = isoparse(d.pop("startDate")).date()

        end_date = isoparse(d.pop("endDate")).date()

        text_reuse_cluster_time_coverage = cls(
            start_date=start_date,
            end_date=end_date,
        )

        return text_reuse_cluster_time_coverage



from typing import Literal, Any, Callable, Iterator, cast

from pandas import DataFrame, json_normalize

from impresso.api_client.api.entities import find_entities, get_entity
from impresso.api_client.models.find_entities_base_find_response import (
    FindEntitiesBaseFindResponse,
)
from impresso.api_client.models.find_entities_order_by import (
    FindEntitiesOrderBy,
    FindEntitiesOrderByLiteral,
)
from impresso.api_client.types import UNSET
from impresso.api_models import BaseFind, EntityDetails, Filter
from impresso.data_container import DataContainer, iterate_pages
from impresso.resources.base import Resource
from impresso.structures import AND, OR
from impresso.util.error import raise_for_error
from impresso.util.filters import and_or_filter, filters_as_protobuf
from impresso.util.py import get_enum_from_literal


class FindEntitiesSchema(BaseFind):
    """Schema for the find entities response."""

    data: list[EntityDetails]


class FindEntitiesContainer(DataContainer):
    """Response of a find call."""

    def __init__(
        self,
        data: FindEntitiesBaseFindResponse,
        pydantic_model: type[FindEntitiesSchema],
        fetch_method: Callable[..., "FindEntitiesContainer"],
        fetch_method_args: dict[str, Any],
        web_app_search_result_url: str | None = None,
    ):
        super().__init__(data, pydantic_model, web_app_search_result_url)
        self._fetch_method = fetch_method
        self._fetch_method_args = fetch_method_args

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            return json_normalize(self._data.to_dict()["data"]).set_index("uid")
        return DataFrame()

    def pages(self) -> Iterator["FindEntitiesContainer"]:
        """Iterate over all pages of results."""
        yield self
        yield from iterate_pages(
            self._fetch_method,
            self._fetch_method_args,
            self.offset,
            self.limit,
            self.total,
        )


class GetEntityContainer(DataContainer):
    """Response of a get call."""

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()
        if len(data):
            return json_normalize([self._data.to_dict()]).set_index("uid")
        return DataFrame()


EntityType = Literal["person", "location"]


class EntitiesResource(Resource):
    """Search entities in the Impresso database.

    Examples:
        Search for entities by name:
        >>> results = entities.find(term="Napoleon")  # doctest: +SKIP
        >>> print(results.df)  # doctest: +SKIP

        Filter entities by type:
        >>> results = entities.find(term="Paris", entity_type="location")  # doctest: +SKIP
        >>> print(results.df)  # doctest: +SKIP

        Get entity details with Wikidata resolution:
        >>> results = entities.find(term="Napoleon", resolve=True)  # doctest: +SKIP
        >>> print(results.df)  # doctest: +SKIP

        Get a specific entity by its ID:
        >>> entity_id = "some-entity-id"  # Replace with a real ID
        >>> entity = entities.get(entity_id)  # doctest: +SKIP
        >>> print(entity.df)  # doctest: +SKIP
    """

    name = "entities"

    def find(
        self,
        term: str | None = None,
        wikidata_id: str | AND[str] | OR[str] | None = None,
        entity_id: str | AND[str] | OR[str] | None = None,
        entity_type: EntityType | AND[EntityType] | OR[EntityType] | None = None,
        order_by: FindEntitiesOrderByLiteral | None = None,
        resolve: bool = False,
        limit: int | None = None,
        offset: int | None = None,
    ) -> FindEntitiesContainer:
        """
        Search entities in Impresso.

        Args:
            term: Search term.
            wikidata_id: Return only entities resolved to this Wikidata ID.
            entity_id: Return only entity with this ID.
            entity_type: Return only entities of this type.
            order_by: Field to order results by.
            resolve: Return Wikidata details of the entities, if the entity is linked to a Wikidata entry.
            limit: Number of results to return.
            offset: Number of results to skip.

        Returns:
            FindEntitiesContainer: Data container with a page of results of the search.
        """

        filters: list[Filter] = []
        if entity_type is not None:
            filters.extend(and_or_filter(entity_type, "type"))  # type: ignore
        if wikidata_id is not None:
            filters.extend(and_or_filter(wikidata_id, "wikidata_id"))
        if entity_id is not None:
            filters.extend(and_or_filter(entity_id, "uid"))

        filters_pb = filters_as_protobuf(filters or [])

        result = find_entities.sync(
            client=self._api_client,
            term=term if term is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, FindEntitiesOrderBy)
                if order_by is not None
                else UNSET
            ),
            limit=limit if limit is not None else UNSET,
            offset=offset if offset is not None else UNSET,
            filters=filters_pb if filters_pb else UNSET,
            resolve=resolve,
        )
        raise_for_error(result)
        return FindEntitiesContainer(
            cast(FindEntitiesBaseFindResponse, result),
            FindEntitiesSchema,
            fetch_method=self.find,
            fetch_method_args={
                "term": term,
                "wikidata_id": wikidata_id,
                "entity_id": entity_id,
                "entity_type": entity_type,
                "order_by": order_by,
                "resolve": resolve,
            },
            web_app_search_result_url=(
                _build_web_app_find_entities_url(
                    base_url=self._get_web_app_base_url(),
                    term=term,
                )
                if wikidata_id is None and entity_type is None
                else None
            ),
        )

    def get(self, id: str) -> GetEntityContainer:
        """Get entity by ID.

        Args:
            id: The ID of the entity to retrieve.

        Returns:
            GetEntityContainer: Data container with the entity information.
        """

        result = get_entity.sync(
            client=self._api_client,
            id=id,
        )
        raise_for_error(result)
        return GetEntityContainer(
            result,
            FindEntitiesSchema,
            web_app_search_result_url=_build_web_app_get_entity_url(
                base_url=self._get_web_app_base_url(),
                id=id,
            ),
        )


def _build_web_app_find_entities_url(
    base_url: str,
    term: str | None = None,
) -> str:
    query_params = {
        "q": term,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    url = f"{base_url}/entities"
    return f"{url}?{query_string}" if query_string else url


def _build_web_app_get_entity_url(
    base_url: str,
    id: str,
) -> str:
    return f"{base_url}/entities/{id}"



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="MediaSourceTotals")


@_attrs_define
class MediaSourceTotals:
    """
    Attributes:
        articles (Union[Unset, int]): The number of articles in the media source.
        issues (Union[Unset, int]): The number of issues in the media source.
        pages (Union[Unset, int]): The number of pages in the media source.
    """

    articles: Union[Unset, int] = UNSET
    issues: Union[Unset, int] = UNSET
    pages: Union[Unset, int] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        articles = self.articles

        issues = self.issues

        pages = self.pages

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if articles is not UNSET:
            field_dict["articles"] = articles
        if issues is not UNSET:
            field_dict["issues"] = issues
        if pages is not UNSET:
            field_dict["pages"] = pages

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        articles = d.pop("articles", UNSET)

        issues = d.pop("issues", UNSET)

        pages = d.pop("pages", UNSET)

        media_source_totals = cls(
            articles=articles,
            issues=issues,
            pages=pages,
        )

        return media_source_totals



from enum import Enum
from typing import Literal


class FilterContext(str, Enum):
    EXCLUDE = "exclude"
    INCLUDE = "include"

    def __str__(self) -> str:
        return str(self.value)


FilterContextLiteral = Literal[
    "exclude",
    "include",
]



from enum import Enum
from typing import Literal


class ImpressoTextEmbeddingRequestSearchTarget(str, Enum):
    MULTIMODAL = "multimodal"
    TEXT = "text"

    def __str__(self) -> str:
        return str(self.value)


ImpressoTextEmbeddingRequestSearchTargetLiteral = Literal[
    "multimodal",
    "text",
]



from enum import Enum
from typing import Literal


class FindCollectionsOrderBy(str, Enum):
    DATE = "date"
    SIZE = "size"
    VALUE_0 = "-date"
    VALUE_2 = "-size"

    def __str__(self) -> str:
        return str(self.value)


FindCollectionsOrderByLiteral = Literal[
    "date",
    "size",
    "-date",
    "-size",
]



from enum import Enum
from typing import Literal


class ContentItemCopyrightStatus(str, Enum):
    EUO = "euo"
    IN_CPY = "in_cpy"
    NKN = "nkn"
    PBL = "pbl"
    UND = "und"
    UNK = "unk"

    def __str__(self) -> str:
        return str(self.value)


ContentItemCopyrightStatusLiteral = Literal[
    "euo",
    "in_cpy",
    "nkn",
    "pbl",
    "und",
    "unk",
]



"""JWT utilities."""

from typing import Literal
import jwt


def get_jwt_status(
    token: str,
) -> tuple[Literal["valid", "expired", "invalid"], dict | None]:
    """
    Validate a JWT token.
    """
    try:
        decoded_token = jwt.decode(
            token,
            options={"verify_signature": False, "verify_exp": True, "verify_iat": True},
        )
        return "valid", decoded_token
    except jwt.ExpiredSignatureError:
        # Signature has expired
        return "expired", None
    except jwt.InvalidTokenError:
        # Token is invalid
        return "invalid", None



from enum import Enum
from typing import Literal


class AuthenticationCreateRequestStrategy(str, Enum):
    JWT_APP = "jwt-app"
    LOCAL = "local"

    def __str__(self) -> str:
        return str(self.value)


AuthenticationCreateRequestStrategyLiteral = Literal[
    "jwt-app",
    "local",
]



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.new_collection_request_access_level import NewCollectionRequestAccessLevel
from ..types import UNSET, Unset

T = TypeVar("T", bound="NewCollectionRequest")


@_attrs_define
class NewCollectionRequest:
    """Create new collection request

    Attributes:
        name (str):
        description (Union[Unset, str]):
        access_level (Union[Unset, NewCollectionRequestAccessLevel]): Access level of the collection.
    """

    name: str
    description: Union[Unset, str] = UNSET
    access_level: Union[Unset, NewCollectionRequestAccessLevel] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        description = self.description

        access_level: Union[Unset, str] = UNSET
        if not isinstance(self.access_level, Unset):
            access_level = self.access_level.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if access_level is not UNSET:
            field_dict["accessLevel"] = access_level

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        description = d.pop("description", UNSET)

        _access_level = d.pop("accessLevel", UNSET)
        access_level: Union[Unset, NewCollectionRequestAccessLevel]
        if isinstance(_access_level, Unset):
            access_level = UNSET
        else:
            access_level = NewCollectionRequestAccessLevel(_access_level)

        new_collection_request = cls(
            name=name,
            description=description,
            access_level=access_level,
        )

        return new_collection_request



import base64
import datetime
from typing import Callable, List, TypeVar

from impresso.api_models import Q, Filter, QItem
from impresso.protobuf import query_pb2 as pb
from impresso.structures import AND, OR, TermSet


def filters_as_protobuf(filters: List[Filter]) -> str | None:
    """
    Convert a list of filters to a protobuf string.
    """
    if filters is None or len(filters) == 0:
        return None

    pb_filters = [_to_pb_filter(f) for f in filters]
    q = pb.SearchQuery(filters=pb_filters)
    return base64.b64encode(q.SerializeToString()).decode("utf-8")


def _to_pb_filter(filter: Filter) -> pb.Filter:
    """
    Convert a Filter to a protobuf Filter.
    """
    filter_dict = filter.model_dump(
        exclude_defaults=True, exclude_none=True, exclude_unset=True
    )

    pb_filter = pb.Filter()

    context = filter_dict.get("context")
    if context is not None:
        pb_filter.context = _to_pb_filter_context(context)

    op = filter_dict.get("op")
    if op is not None:
        pb_filter.op = _to_pb_filter_operator(op)

    filter_type = filter_dict.get("type")
    if filter_type is not None:
        pb_filter.type = _to_pb_filter_type(filter_type)

    precision = filter_dict.get("precision")
    if precision is not None:
        pb_filter.precision = _to_pb_filter_precision(precision)

    q = filter_dict.get("q")
    if q is not None:
        pb_filter.q.extend([q] if isinstance(q, str) else q)

    daterange = filter_dict.get("daterange")
    if daterange is not None:
        daterange_from, daterange_to = daterange.split(" TO ")
        if daterange_from is not None and daterange_to is not None:
            _from = int(datetime.datetime.fromisoformat(daterange_from).timestamp())
            _to = int(datetime.datetime.fromisoformat(daterange_to).timestamp())

            pb_filter.daterange = _to_pb_filter_daterange(_from, _to)

    return pb_filter


def _to_pb_filter_context(context: str) -> pb.FilterContext:
    v = getattr(pb.FilterContext, "CONTEXT_" + context.upper())
    if v is None:
        raise ValueError(f"Invalid filter context: {context}")
    return v


def _to_pb_filter_operator(op: str) -> pb.FilterOperator:
    v = getattr(pb.FilterOperator, "OPERATOR_" + op.upper())
    if v is None:
        raise ValueError(f"Invalid filter operator: {op}")
    return v


def _to_pb_filter_type(type: str) -> pb.FilterType:
    v = getattr(pb.FilterType, "TYPE_" + type.upper())
    if v is None:
        raise ValueError(f"Invalid filter type: {type}")
    return v


def _to_pb_filter_precision(p: str) -> pb.FilterPrecision:
    v = getattr(pb.FilterPrecision, "PRECISION_" + p.upper())
    if v is None:
        raise ValueError(f"Invalid filter precision: {p}")
    return v


def _to_pb_filter_daterange(from_: int, to: int) -> pb.DateRange:
    return pb.DateRange(to=to, **{"from": from_})


T = TypeVar("T")


def and_or_filter(
    item: T | AND[T] | OR[T],
    type: str,
    converter: Callable[[T], str] = lambda x: str(x),
) -> list[Filter]:
    if isinstance(item, TermSet):
        filter = Filter(
            type=type,
            q=list([QItem(converter(i)) for i in item]),
            op=item.op,
            context="exclude" if item.inverted else "include",
            precision=item.precision,
            daterange=None,
        )
        chain_filters = [
            Filter(
                type=type,
                q=list([QItem(converter(i)) for i in chain_item]),
                op=chain_item.op,
                context="exclude" if chain_item.inverted else "include",
                daterange=None,
            )
            for chain_item in item.chain
        ]
        return [filter] + chain_filters
    else:
        return [
            Filter(type=type, q=Q(converter(item)), daterange=None, precision="exact")
        ]



from enum import Enum
from typing import Literal


class FilterPrecision(str, Enum):
    EXACT = "exact"
    FUZZY = "fuzzy"
    PARTIAL = "partial"
    SOFT = "soft"

    def __str__(self) -> str:
        return str(self.value)


FilterPrecisionLiteral = Literal[
    "exact",
    "fuzzy",
    "partial",
    "soft",
]



from impresso.api_client.client import AuthenticatedClient


class Resource:
    """Base for all API backed resources."""

    name: str

    def __init__(self, api_client: AuthenticatedClient):
        self._api_client = api_client

    def _get_web_app_base_url(self) -> str:
        return "https://impresso-project.ch/app"


DEFAULT_PAGE_SIZE = 100



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="ImpressoNamedEntityRecognitionEntityConfidence")


@_attrs_define
class ImpressoNamedEntityRecognitionEntityConfidence:
    """
    Attributes:
        ner (Union[Unset, float]): Confidence score for the named entity recognition
        nel (Union[Unset, float]): Confidence score for the named entity linking
    """

    ner: Union[Unset, float] = UNSET
    nel: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        ner = self.ner

        nel = self.nel

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if ner is not UNSET:
            field_dict["ner"] = ner
        if nel is not UNSET:
            field_dict["nel"] = nel

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        ner = d.pop("ner", UNSET)

        nel = d.pop("nel", UNSET)

        impresso_named_entity_recognition_entity_confidence = cls(
            ner=ner,
            nel=nel,
        )

        return impresso_named_entity_recognition_entity_confidence



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.error import Error
from ...models.freeform import Freeform
from ...models.interact_with_experiment_body import InteractWithExperimentBody
from ...types import Response


def _get_kwargs(
    id: str,
    *,
    body: InteractWithExperimentBody,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "put",
        "url": f"/experiments/{id}",
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Error, Freeform]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Freeform.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = Error.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = Error.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = Error.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.IM_A_TEAPOT:
        response_418 = Error.from_dict(response.json())

        return response_418
    if response.status_code == HTTPStatus.UNPROCESSABLE_CONTENT:
        response_422 = Error.from_dict(response.json())

        return response_422
    if response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
        response_429 = Error.from_dict(response.json())

        return response_429
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = Error.from_dict(response.json())

        return response_500
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Error, Freeform]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    body: InteractWithExperimentBody,
) -> Response[Union[Error, Freeform]]:
    """Interact with an experiment with experiment specific data

    Args:
        id (str):
        body (InteractWithExperimentBody): Experiment specific request body

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Error, Freeform]]
    """

    kwargs = _get_kwargs(
        id=id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: str,
    *,
    client: AuthenticatedClient,
    body: InteractWithExperimentBody,
) -> Optional[Union[Error, Freeform]]:
    """Interact with an experiment with experiment specific data

    Args:
        id (str):
        body (InteractWithExperimentBody): Experiment specific request body

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Error, Freeform]
    """

    return sync_detailed(
        id=id,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    body: InteractWithExperimentBody,
) -> Response[Union[Error, Freeform]]:
    """Interact with an experiment with experiment specific data

    Args:
        id (str):
        body (InteractWithExperimentBody): Experiment specific request body

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Error, Freeform]]
    """

    kwargs = _get_kwargs(
        id=id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: str,
    *,
    client: AuthenticatedClient,
    body: InteractWithExperimentBody,
) -> Optional[Union[Error, Freeform]]:
    """Interact with an experiment with experiment specific data

    Args:
        id (str):
        body (InteractWithExperimentBody): Experiment specific request body

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Error, Freeform]
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            body=body,
        )
    ).parsed



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.collection import Collection
from ...models.error import Error
from ...models.new_collection_request import NewCollectionRequest
from ...types import Response


def _get_kwargs(
    id: str,
    *,
    body: NewCollectionRequest,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "patch",
        "url": f"/collections/{id}",
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Collection, Error]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Collection.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = Error.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = Error.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = Error.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.IM_A_TEAPOT:
        response_418 = Error.from_dict(response.json())

        return response_418
    if response.status_code == HTTPStatus.UNPROCESSABLE_CONTENT:
        response_422 = Error.from_dict(response.json())

        return response_422
    if response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
        response_429 = Error.from_dict(response.json())

        return response_429
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = Error.from_dict(response.json())

        return response_500
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Collection, Error]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    body: NewCollectionRequest,
) -> Response[Union[Collection, Error]]:
    """Update a collection

    Args:
        id (str):
        body (NewCollectionRequest): Create new collection request

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Collection, Error]]
    """

    kwargs = _get_kwargs(
        id=id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: str,
    *,
    client: AuthenticatedClient,
    body: NewCollectionRequest,
) -> Optional[Union[Collection, Error]]:
    """Update a collection

    Args:
        id (str):
        body (NewCollectionRequest): Create new collection request

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Collection, Error]
    """

    return sync_detailed(
        id=id,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    body: NewCollectionRequest,
) -> Response[Union[Collection, Error]]:
    """Update a collection

    Args:
        id (str):
        body (NewCollectionRequest): Create new collection request

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Collection, Error]]
    """

    kwargs = _get_kwargs(
        id=id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: str,
    *,
    client: AuthenticatedClient,
    body: NewCollectionRequest,
) -> Optional[Union[Collection, Error]]:
    """Update a collection

    Args:
        id (str):
        body (NewCollectionRequest): Create new collection request

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Collection, Error]
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            body=body,
        )
    ).parsed
